/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.tool;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import gluonj.NotFoundException;
import gluonj.parser.ParseException;
import gluonj.ui.StaticWeaver;
import gluonj.ui.StaticWeaver;
import gluonj.weaver.CannotCompileException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

public class GjcTask extends Task {

    public void execute() throws BuildException {

        StringBuffer buf = new StringBuffer();
        for (Iterator iter = options.iterator(); iter.hasNext();) {
            String token = (String) iter.next();
            buf.append(token + " ");
        }
        for (Iterator iter = fileSets.iterator(); iter.hasNext();) {
            FileSet fs = (FileSet) iter.next();
            DirectoryScanner scanner = fs.getDirectoryScanner(getProject());
            String[] files = scanner.getIncludedFiles();

            for (int i = 0; i < files.length; i++) {
                buf.append(files[i] + " ");
            }
        }
        try {
            System.out.println(buf);
            StaticWeaver.weave(buf.toString().split(" "));
        } catch (Exception e) {
            e.printStackTrace(System.err);
            throw new BuildException(e);
        }
    }

    private List options = new LinkedList();

    public void setClassDir(String path) {
        addOpt(StaticWeaver.opt_src, path);
    }

    public void setInjar(String path) {
        addOpt(StaticWeaver.opt_injar, path);
    }

    public void setIncludes(String path) {
        addOpt(StaticWeaver.opt_incl, path);
    }

    public void setClassPath(String path) {
        addOpt(StaticWeaver.opt_cp, path);
    }

    public void setOutjar(String path) {
        addOpt(StaticWeaver.opt_outjar, path);
    }

    public void setDest(String path) {
        addOpt(StaticWeaver.opt_dest, path);
    }

    private void addOpt(String key, String value) {
        options.add("-" + key);
        options.add(value);
    }

    private List fileSets = new LinkedList();

    public FileSet createFileSet() {
        FileSet s = new FileSet();
        fileSets.add(s);
        return s;
    }
}