/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package gluonj.deployment;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.security.CodeSource;
import java.security.ProtectionDomain;
import java.security.cert.Certificate;

import org.jboss.mx.loading.RepositoryClassLoader;
import org.jboss.util.loading.Translator;

import sun.misc.Resource;
import sun.misc.URLClassPath;

public abstract class BugFixedTranslator implements Translator {

    final public byte[] transform(ClassLoader cl, String name, Class clazz,
            ProtectionDomain domain, byte[] bytecode) throws Exception {

        setCorrectProtectionDomain(cl, name, clazz, domain);
        return transform0(cl, name, clazz, domain, bytecode);

    }

    abstract public byte[] transform0(ClassLoader cl, String name, Class clazz,
            ProtectionDomain domain, byte[] bytecode) throws Exception;

    /**
     * This method make the correct ProtectionDomain object. When
     * RepositoryClassLoader does not have a Translator object, this task is
     * done by URLClassLoder. However, when RepositoryClassLoader has a
     * Translator object, this task is not done (skipped). So this task must be
     * done explicitly by Translator.
     */

    private void setCorrectProtectionDomain(ClassLoader cl, String name,
            Class clazz, ProtectionDomain domain) throws Exception {

        ProtectionDomain newdomain = getCorrectProtectionDomain(cl, name);
        /*
         * BEGIN domain.codeSource = newdomain.codeSource;
         */
        Class cProtectionDomain = Class
                .forName("java.security.ProtectionDomain");
        Field fCodesource = cProtectionDomain.getDeclaredField("codesource");
        fCodesource.setAccessible(true);
        fCodesource.set(domain, fCodesource.get(newdomain));
        /*
         * END
         */
    }

    private ProtectionDomain getCorrectProtectionDomain(ClassLoader cl,
            String name) throws Exception {

        String s = name.replace('.', '/').concat(".class");
        /*
         * BEGIN URLClassPath ucp = cl.ucp;
         */
        Class uclc = Class.forName("java.net.URLClassLoader");
        Field fUcp = uclc.getDeclaredField("ucp");
        fUcp.setAccessible(true);
        URLClassPath ucp = (URLClassPath) fUcp.get(cl);
        /*
         * END
         */
        Resource resource = ucp.getResource(s, false);
        URL url = resource.getCodeSourceURL();
        resource.getBytes(); // this method creates certificates.
        Certificate[] acertificate = resource.getCertificates();
        CodeSource codesource = new CodeSource(url, acertificate);
        /*
         * BEGIN ProtectionDomain newdomain =
         * rcl.getProtectionDomain(codesource);
         */
        Class cSecureClassLoader = Class
                .forName("java.security.SecureClassLoader");

        Method[] ms = cSecureClassLoader.getDeclaredMethods();
        Method mGetProtectionDomain = cSecureClassLoader.getDeclaredMethod(
                "getProtectionDomain", new Class[] { Class
                        .forName("java.security.CodeSource"), });

        mGetProtectionDomain.setAccessible(true);
        ProtectionDomain newdomain = (ProtectionDomain) mGetProtectionDomain
                .invoke(cl, new Object[] { codesource });
        /*
         * END
         */

        return newdomain;
    }
}