/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package gluonj.deployment;

import java.io.IOException;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;
import java.util.regex.Pattern;

public class FileFinderInJar {

    private JarInputStream jar;

    private URL jarURL;

    public FileFinderInJar(URL jarURL) throws IOException {
        this.jarURL = jarURL;
        jar = new JarInputStream(jarURL.openStream());
    }

    /*
     * returns all filenames-in-jar matched with pattern. pattern is like
     * usr/test/Test.java, usr/test/*.java, usr/** /build.xml
     */

    public String[] getFileNames(String pattern) throws IOException {

        List matchedNames = new LinkedList();
        JarEntry entry;
        while ((entry = jar.getNextJarEntry()) != null) {
            String entryName = entry.getName();
            if (match(pattern, entryName)) {
                matchedNames.add(entryName);
            }

        }
        return (String[]) matchedNames.toArray(new String[matchedNames.size()]);
    }

    public URL[] getFileURL(String pattern) throws IOException {
        String[] fileNames = getFileNames(pattern);

        URL[] fileURL = new URL[fileNames.length];

        for (int i = 0; i < fileURL.length; i++) {
            fileURL[i] = new URL("jar:" + jarURL + "!/" + fileNames[i]);
        }
        return fileURL;
    }

    /*
     * format of pattern is
     * 
     * "*" means more than zero characters except SEPARATOR.
     * 
     * "**" means more than zero characters. this wildcard must exist between
     * two SEPARATORs. TestExec: "usr/test/** /Main"
     * 
     * a targetName is like usr/test/Test, usr/test/Main$1, usr/test/Main$Bar
     * 
     * a pattern is like follows:
     * 
     * usr/* /Test (first 1 name matches with)
     * 
     * usr/test/Main* (last 2 names match with)
     * 
     * usr/** /* (all names match with)
     * 
     * 
     * 
     * "*" -> "[^/]*" "/**" + "/ " -> "/.*" + "/ | /"
     * 
     * (algorithm)
     * 
     * "*" -> "[^/]*" "/[^/]*[^/]*" + "/ " -> "(/.*" + "/ | /)"
     * 
     * "." -> "\." "(" -> "\(" ")" -> "\)" "^" -> "\^" "+" -> "\+" "{" -> "\{"
     * "}" -> "\}"
     * 
     *  
     */

    private boolean match(String pattern, String targetName) {

        pattern = pattern.replaceAll("\\*", "[^/]*");
        pattern = pattern.replaceAll("\\/\\[\\^\\/\\]\\*\\[\\^\\/\\]\\*\\/",
                "(/.*/|/)");

        return Pattern.matches(pattern, targetName);
    }
}