/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast;

import gluonj.ast.java.ASTBlock;
import gluonj.ast.java.ASTFormalParameter;
import gluonj.ast.joinpoint.JoinPoint;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.weaver.JoinPointConstants;

import java.util.List;

public class Advice implements JoinPointConstants {

    ASTFormalParameter[] parameter; /* advice parameter */

    AdviceType type; /* BEFORE, AFTER, AROUND */

    ASTPointcutExpr root; /* pointcut */

    ASTBlock body; /* java code */

    public Advice(List param, AdviceType type, ASTPointcutExpr root,
            ASTBlock body) {

        parameter = (ASTFormalParameter[]) param
                .toArray(new ASTFormalParameter[param.size()]);
        this.type = type;
        this.root = root;
        this.body = body;
    }

    public Advice(ASTFormalParameter[] param, AdviceType type,
            ASTPointcutExpr root, ASTBlock body) {

        parameter = param;
        this.type = type;
        this.root = root;
        this.body = body;
    }

    public String toString() {
        StringBuffer str = new StringBuffer();
        str.append("<advice>");
        str.append("<type>" + type.toString() + "</type>");

        for (int i = 0; i < parameter.length; i++) {
            ASTFormalParameter element = parameter[i];
            str.append("<param>" + element.toString() + "</param>");
        }
        str.append("<pointcut>\n" + root.toString() + "</pointcut>\n");
        str.append("<body>\n" + body.toString() + "</body>\n");
        str.append("</advice>\n");

        return str.toString();
    }

    public ASTFormalParameter[] getAllParameter() {
        return parameter;
    }

    public AdviceType getType() {
        return type;
    }

    public ASTBlock getBody() {
        return body;
    }

    public ASTPointcutExpr getPointcutExpr() {
        return root;
    }

    /**
     * returns information of the advice. if the advice does not match JoinPoint
     * jp, this method returns null.
     */
    public AdviceInfo match(JoinPoint jp) {

        return AdviceInfoImpl.factory(this, jp);
    }

    private Import[] packages = new Import[0];

    public static Advice setPackage(Advice advice, Import[] packages) {
        Advice newAdv = new Advice(advice.parameter, advice.type, advice.root,
                advice.body);
        newAdv.packages = packages;
        return newAdv;
    }
    public Import[] getPackage() {
        return packages;
    }
}