/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast;

import gluonj.ast.intertype.IntertypeDecl;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/*
 * aspect := anonymous-field-declaration* intertype-declaration*
 * pointcut-declaration* advice*
 *  
 */
public class Aspect {

    private List imports;

    private List intertypeDecl;

    private List pointcutDecl;

    private List advice;

    private String name;

    Aspect() {
    }

    public Aspect(List m, List i, List p, List a) {
        imports = m;
        intertypeDecl = i;
        pointcutDecl = p;
        advice = a;
    }

    public Aspect(List m, List i, List p, List a, String name) {
        this(m, i, p, a);
        this.name = name;
    }

    public Aspect(Import[] m, IntertypeDecl[] i, PointcutDecl[] p, Advice[] a) {
        imports = Arrays.asList(m);
        intertypeDecl = Arrays.asList(i);
        pointcutDecl = Arrays.asList(p);
        advice = Arrays.asList(a);
    }

    public String toString() {

        StringBuffer str = new StringBuffer();
        List[] member = new List[] { imports, intertypeDecl, pointcutDecl,
                advice };
        str.append("<aspect>\n");
        for (int i = 0; i < member.length; i++) {
            for (Iterator iter = member[i].iterator(); iter.hasNext();) {
                str.append(iter.next().toString());
            }
            str.append("\n");
        }
        str.append("</aspect>\n");
        return str.toString();
    }

    public IntertypeDecl[] getIntertypeDecl() {
        IntertypeDecl[] decl = new IntertypeDecl[intertypeDecl.size()];
        intertypeDecl.toArray(decl);
        return decl;
    }

    public PointcutDecl[] getPointcutDecl() {
        PointcutDecl[] decl = new PointcutDecl[pointcutDecl.size()];
        pointcutDecl.toArray(decl);
        return decl;
    }

    public Advice[] getAdvice() {
        return (Advice[]) advice.toArray(new Advice[advice.size()]);
    }

    public Import[] getImport() {
        return (Import[]) imports.toArray(new Import[imports.size()]);
    }

    public String getName() {
        return name;
    }

    
    public static Aspect setImport(Aspect aspect, Import[] newImport) {
        Aspect newAspect = new Aspect(Arrays.asList(newImport), aspect.intertypeDecl,
                aspect.pointcutDecl, aspect.advice, aspect.name);
        return newAspect;
    }
    
    public Aspect replaceImportWith(Import[] newImport) {
        Aspect newAspect = new Aspect(Arrays.asList(newImport), intertypeDecl,
                pointcutDecl, advice, name);
        newAspect.name = name;
        return newAspect;
    }

    public Aspect replaceIntertypeDeclWith(IntertypeDecl[] newDecl) {
        Aspect newAspect = new Aspect(imports, Arrays.asList(newDecl),
                pointcutDecl, advice, name);
        return newAspect;
    }

    public Aspect replacePointcutDeclWith(PointcutDecl[] newDecl) {
        Aspect newAspect = new Aspect(imports, intertypeDecl, Arrays
                .asList(newDecl), advice, name);
        return newAspect;
    }

    public Aspect replaceAdviceWith(Advice[] newAdvice) {
        Aspect newAspect = new Aspect(imports, intertypeDecl, pointcutDecl,
                Arrays.asList(newAdvice), name);
        return newAspect;
    }

    public Aspect replaceNameWith(String newName) {
        Aspect newAspect = new Aspect(imports, intertypeDecl, pointcutDecl,
                advice, newName);
        return newAspect;
    }
}