/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.intertype;

import gluonj.ast.java.ASTBehavior;
import gluonj.ast.java.ASTConstructor;
import gluonj.ast.java.ASTMethod;
import gluonj.ast.java.JavaVisitor;
import gluonj.ast.joinpoint.JPClass;
import gluonj.ast.joinpoint.JoinPoint;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.CtNewConstructor;
import javassist.CtNewMethod;
import javassist.NotFoundException;

public class BehaviorAdditionSig extends ITDeclSig {

    ASTBehavior behavior;

    BehaviorAdditionSig(ASTBehavior b) {
        behavior = b;
    }

    public void addDeclTo(final JoinPoint jp) throws gluonj.weaver.CannotCompileException {
        exc = null;
        behavior.accept(new JavaVisitor() {
            public void atASTMethod(ASTMethod method) {
                try {
                    ClassPool pool = jp.getPool();
                    CtClass cc = ((JPClass) jp).getCtClass();
                    int modifier = method.getModifier().getModifiers();
                    CtClass returnType;
                    returnType = pool.get(method.getReturnType().getType());

                    String name = method.getName();
                    CtClass[] parameter = new CtClass[method.getParameter().length];
                    CtClass[] throwable = new CtClass[method.getThrowable().length];

                    for (int i = 0; i < parameter.length; i++) {
                        parameter[i] = pool.get(method.getParameter()[i]
                                .getType().getType());
                    }
                    for (int i = 0; i < throwable.length; i++) {
                        throwable[i] = pool.get(method.getThrowable()[i]
                                .getType());
                    }

                    CtMethod cm = CtNewMethod.abstractMethod(returnType, name,
                            parameter, throwable, cc);

                    cm.setModifiers(modifier);
                    cc.addMethod(cm);

                } catch (NotFoundException e) {
                    exc = new gluonj.weaver.CannotCompileException(e);
                } catch (CannotCompileException e) {
                    exc = new gluonj.weaver.CannotCompileException(
                            "An method body declared as inter-type declaration might have an error.",
                            e);
                }
            }

            public void atASTConstructor(ASTConstructor constructor) {
                try {
                    ClassPool pool = jp.getPool();
                    CtClass cc = ((JPClass) jp).getCtClass();
                    int modifier = constructor.getModifier().getModifiers();
                    CtClass[] parameter = new CtClass[constructor
                            .getParameter().length];
                    CtClass[] throwable = new CtClass[constructor
                            .getThrowable().length];
                    for (int i = 0; i < parameter.length; i++) {
                        parameter[i] = pool.get(constructor.getParameter()[i]
                                .getType().getType());
                    }
                    for (int i = 0; i < throwable.length; i++) {
                        throwable[i] = pool.get(constructor.getThrowable()[i]
                                .getType());
                    }
                    CtConstructor ccn = CtNewConstructor.skeleton(parameter,
                            throwable, cc);

                    ccn.setModifiers(modifier);
                    cc.addConstructor(ccn);

                } catch (NotFoundException e) {
                    exc = new gluonj.weaver.CannotCompileException(e);
                } catch (CannotCompileException e) {
                    exc = new gluonj.weaver.CannotCompileException(
                            "An constructor body declared as inter-type declaration might have an error.",
                            e);
                }
            }
        });
        if(exc != null) {
            throw exc;
        }
    }

    private gluonj.weaver.CannotCompileException exc;

    public void addBodyTo(final JoinPoint jp)
            throws gluonj.weaver.CannotCompileException {

        exc = null;

        behavior.accept(new JavaVisitor() {
            public void atASTConstructor(ASTConstructor constructor) {
                try {
                    ClassPool pool = jp.getPool();
                    CtClass cc = ((JPClass) jp).getCtClass();
                    CtClass[] parameter = new CtClass[constructor
                            .getParameter().length];
                    for (int i = 0; i < parameter.length; i++) {
                        parameter[i] = pool.get(constructor.getParameter()[i]
                                .getType().getType());
                    }
                    CtConstructor ccn = cc.getDeclaredConstructor(parameter);
                    ccn.setBody(constructor.getBody().toString());
                } catch (NotFoundException e) {
                    exc = new gluonj.weaver.CannotCompileException(e);
                } catch (CannotCompileException e) {
                    exc = new gluonj.weaver.CannotCompileException(
                            "An constructor body declared as inter-type declaration might have an error.",
                            e);
                }
            }

            public void atASTMethod(ASTMethod method) {
                try {
                    ClassPool pool = jp.getPool();
                    CtClass cc = ((JPClass) jp).getCtClass();
                    String name = method.getName();
                    CtClass[] parameter = new CtClass[method.getParameter().length];
                    for (int i = 0; i < parameter.length; i++) {
                        parameter[i] = pool.get(method.getParameter()[i]
                                .getType().getType());
                    }
                    CtMethod cm = cc.getDeclaredMethod(name, parameter);
                    cm.setBody(method.getBody().toString());
                } catch (NotFoundException e) {
                    exc = new gluonj.weaver.CannotCompileException(e);
                } catch (CannotCompileException e) {
                    exc = new gluonj.weaver.CannotCompileException(
                            "An method body declared as inter-type declaration might have an error.",
                            e);
                }
            }
        });

        if (exc != null) {
            throw exc;
        }
    }
}