/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pattern;

import java.util.HashMap;

public class PrimitiveTypeNamePattern extends BasicTypeNamePattern {

	public static final int BOOLEAN = 1;

	public static final int BYTE = 2;

	public static final int CHAR = 4;

	public static final int DOUBLE = 8;

	public static final int FLOAT = 16;

	public static final int INT = 32;

	public static final int LONG = 64;

	public static final int SHORT = 128;

	public static final int VOID = 256;

	int type;

	static HashMap intMap;
	static HashMap strMap;

	static Object[][] primitiveIdx =
		new Object[][] { { "boolean", new Integer(BOOLEAN)}, {
			"byte", new Integer(BYTE)
		}, {
			"char", new Integer(CHAR)
		}, {
			"double", new Integer(DOUBLE)
		}, {
			"float", new Integer(FLOAT)
		}, {
			"int", new Integer(INT)
		}, {
			"long", new Integer(LONG)
		}, {
			"short", new Integer(SHORT)
		}, {
			"void", new Integer(VOID)
		}, };

	static {
		intMap = new HashMap();
		strMap = new HashMap();
		for (int i = 0; i < primitiveIdx.length; i++) {
			intMap.put(primitiveIdx[i][0], primitiveIdx[i][1]);
			strMap.put(primitiveIdx[i][1], primitiveIdx[i][0]);
		}
	}
	public PrimitiveTypeNamePattern(int idx) {
		type = idx;
	}
	public PrimitiveTypeNamePattern(String str) {
		type = strToInt(str);
	}


	public boolean match(int typeid) {
		return type == typeid;
	}
	public boolean match(String typename) {
		return match(strToInt(typename));
	}

	public int strToInt(String typename) {
		if (intMap.containsKey(typename)) {
			return ((Integer) intMap.get(typename)).intValue();
		} else {
			return -1;
		}
	}

	public String toString() {
		return (String) strMap.get(new Integer(type));
	}
	
	public String getSoleClassName() {

		return toString();
	}
	
	public void accept(TypeNamePatternVisitor v) {
		v.atPrimitiveTypeName(this);
	}
}
