/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPBehavior;
import gluonj.ast.joinpoint.JoinPoint;
import gluonj.weaver.UnSupportedException;
import javassist.CtBehavior;
import javassist.CtClass;
import javassist.CtMember;
import javassist.NotFoundException;

/*
 * \bhARXgN^w肷p^[ Ex. (void || int) usr..test.*.test*(int, String, ..)
 * throws !IOException
 *  
 */
public class PcsBehavior extends PcsMember {

    public static final PcsBehavior ANY = new PcsBehavior(new PcsModifier(),
            PcsType.ANY_TYPE, new PcsParameter[] { new PcsParameterDotDot() },
            new PcsType[0]);

    public PcsBehavior(PcsModifier mod, PcsType declaredclass,
            PcsParameter[] param, PcsType[] th) {

        super(mod, declaredclass);
        parameter = param;
        throwable = th;
    }

    PcsBehavior() {
        super();
    }

    PcsParameter[] parameter; //List<ParameterPattern>

    PcsType[] throwable; //List<TypePattern>

    public PcsParameter[] getParameter() {
        return parameter;
    }

    public PcsType[] getThrowable() {
        return throwable;
    }

    public String toString() {
        return modifiers + " " + declaredClass + ".<any>("
                + listToString(parameter) + ") throws"
                + listToString(throwable) + ")";
    }

    protected String listToString(Object[] l) {
        StringBuffer buf = new StringBuffer();
        buf.append("");
        for (int i = 0; i < l.length; i++) {
            buf.append(l[i] + ",");
        }
        buf.append("");
        return buf.toString();
    }

    public void accept(PcsVisitor v) {
        v.atBehaviorPattern(this);
    }

    public boolean match(JoinPoint jp) {

        if (!super.match(jp)) {
            return false;
        }

        if (jp.getJPType() == JoinPoint.CONSTRUCTOR
                || jp.getJPType() == JoinPoint.METHOD) {
            return match(((JPBehavior) jp).getCtBehavior());
        } else {
            return false;
        }
    }

    boolean match(CtBehavior cb) {
        if (!super.match(cb)) {
            return false;
        } else {
            try {
                boolean b1 = matchParameter(cb.getParameterTypes(), parameter);
                boolean b2 = matchThrowable(cb.getExceptionTypes(), throwable);
                return b1 && b2;
            } catch (NotFoundException e) {
                onNotFound(e);
                return false;
            }
        }
    }

    boolean match(CtMember cm) {
        if (super.match(cm)) {
            if (cm instanceof CtBehavior) {
                return match((CtBehavior) cm);
            } else {
                return true;
            }
        } else {
            return false;
        }
    } /*
       * patterns Ŏw肳ꂽNXSclasses Ɋ܂܂Ȃtrue
       */

    private boolean matchThrowable(CtClass[] classes, PcsType[] patterns) {

        for (int i = 0; i < patterns.length; i++) {
            if (!containOf(patterns[i], classes)) {
                return false;
            }
        }
        return true;
    }

    private boolean containOf(PcsType pat, CtClass[] classes) {

        for (int j = 0; j < classes.length; j++) {
            if (pat.match(classes[j])) {
                return true;
            }
        }
        return false;
    }

    private boolean matchParameter(CtClass[] classes, PcsParameter[] patterns) {

        int ddIdx = -1;
        for (int i = 0; i < patterns.length; i++) {
            if (patterns[i] instanceof PcsParameterDotDot) {
                if (ddIdx == -1) {
                    ddIdx = i;
                } else {
                    throw new UnSupportedException(
                            "this pattern has more than two \"..\" (compiler limitation)");
                }
            }
        }
        if (ddIdx == -1) {
            // the pattern has no "..".
            if (classes.length != patterns.length) {
                return false;
            }
            for (int i = 0; i < classes.length; i++) {
                if (!((PcsParameterType) patterns[i]).getPattern().match(
                        classes[i])) {
                    return false;
                }
            }
            return true;
        } else {
            // the pattern has one "..".

            if (classes.length < patterns.length - 1) {
                return false;
            }
            int formerlength = ddIdx;
            int latterlength = patterns.length - 1 - ddIdx;

            //matching the former of parameter patterns
            for (int i = 0; i < formerlength; i++) {
                CtClass cls = classes[i];
                PcsType pat = ((PcsParameterType) patterns[i]).getPattern();
                if (!pat.match(cls)) {
                    return false;
                }
            }
            //matching the latter of parameter patterns
            for (int i = latterlength; i > 0; i--) {

                CtClass cls = classes[classes.length - i];
                PcsType pat = ((PcsParameterType) patterns[patterns.length - i])
                        .getPattern();
                if (!pat.match(cls)) {
                    return false;
                }
            }
            return true;
        }
    }
}
