/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPMethodCall;
import gluonj.ast.joinpoint.JPNewExpr;
import gluonj.ast.joinpoint.JoinPoint;
import javassist.NotFoundException;

public class PcsCall extends PcsExpr {

    PcsBehavior callee;

    public PcsCall(PcsBehavior callee) {
        super(PcsBehavior.ANY);
        this.callee = callee;
    }

    public PcsCall(PcsBehavior where, PcsBehavior callee) {
        super(where);
        this.callee = callee;
    }

    PcsCall() {
        super();
    }

    public PcsBehavior getCallee() {
        return callee;
    }

    public String toString() {
        return "call(" + getCallee() + ") &&" + super.toString();
    }

    public void accept(PcsVisitor v) {
        v.atCallPattern(this);
    }

    public boolean match(JoinPoint jp) {

        if (super.match(jp) && jp.getJPType() == JoinPoint.METHOD_CALL) {
            
            	if(callee instanceof PcsBehavior) {
            	    
            	}
                try {
                    return callee.match(((JPMethodCall) jp)
                            .getMethodCall().getMethod());
                } catch (NotFoundException e) {
                    onNotFound(e);
                    return false;
                }
         //   }
        } else if (super.match(jp) && jp.getJPType() == JoinPoint.NEW_EXPR) {
            if (callee instanceof PcsConstructor) {
                try {
                    PcsConstructor cst = (PcsConstructor) callee;
                    return cst.match(((JPNewExpr) jp).getNewExpr()
                            .getConstructor());
                } catch (NotFoundException e) {
                    onNotFound(e);
                }
            }
            return false;
        } else {
            return false;
        }
    }
}