/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPExpr;
import gluonj.ast.joinpoint.JoinPoint;
import javassist.CtBehavior;

public class PcsExpr extends PointcutShadow {

    public PcsExpr(PcsBehavior where) {
        this.where = where;
    }

    PcsExpr() {
    }

    PcsBehavior where;

    public PcsBehavior getWhere() {
        return where;
    }

    public String toString() {
        return " where(" + where + ")";
    }

    public void accept(PcsVisitor v) {
        v.atExpr(this);
    }

    public boolean match(JoinPoint jp) {
        int type = jp.getJPType();
        if (isExpr(type)) {
            CtBehavior cb = ((JPExpr) jp).getExpr().where();
            return where.match(cb);
        } else {
            return false;
        }
    }

    private boolean isExpr(int t) {
        return t == JoinPoint.FIELD_ACCESS || t == JoinPoint.HANDLER
                || t == JoinPoint.INSTANCEOF || t == JoinPoint.METHOD_CALL
                || t == JoinPoint.NEW_ARRAY || t == JoinPoint.NEW_EXPR;
    }
}
