/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPField;
import gluonj.ast.joinpoint.JoinPoint;
import gluonj.ast.pattern.IdPattern;
import javassist.CtField;
import javassist.NotFoundException;

public class PcsField extends PcsMember {

	public static final PcsField ANY =
		new PcsField(
			new PcsModifier(),
			PcsType.ANY_TYPE,
			PcsType.ANY_TYPE,
			new IdPattern(".*"));

	public PcsField(
		PcsModifier mod,
		PcsType t,
		PcsType declaredclass,
		IdPattern n) {

		super(mod, declaredclass);
		type = t;
		name = n;

	}

	PcsType type;

	IdPattern name;

	PcsField() {
	}

	public void accept(PcsVisitor v) {
		v.atFieldPattern(this);
	}

	public PcsType getType() {
		return type;
	}

	public IdPattern getName() {
		return name;
	}

	public String toString() {
		return modifiers + " " + type + " " + declaredClass + " " + name;
	}

	public boolean match(JoinPoint jp) {
		if (super.match(jp) && jp.getJPType() == JoinPoint.FIELD) {
			return match(((JPField) jp).getCtField());
		} else {
			return false;
		}
	}

	boolean match(CtField cf) {
		try {
			boolean b1 = type.match(cf.getType());
			boolean b2 = name.match(cf.getName());
			return b1 && b2;
		} catch (NotFoundException e) {
			onNotFound(e);
			return false;
		}
	}

}
