/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPMember;
import gluonj.ast.joinpoint.JoinPoint;
import javassist.CtMember;


public abstract class PcsMember extends PointcutShadow {

	public PcsMember(PcsModifier mod, PcsType declaredclass) {
		modifiers = mod;
		declaredClass  = declaredclass;
	}
	PcsMember() {
		super();
	}

	PcsType declaredClass;
  	PcsModifier modifiers;

	public PcsType getDeclaredClass() {
		return declaredClass;
	}

	public PcsModifier getModifier() {
		return modifiers;
	}
	
	public String toString() {
		return "declared:" + declaredClass + " modifiers:" + modifiers;
	}
	public boolean match(JoinPoint jp) {
	    int type = jp.getJPType();
	    
	    if(type == JoinPoint.METHOD || type == JoinPoint.CONSTRUCTOR
	            || type == JoinPoint.FIELD) {
	        CtMember cm = ((JPMember) jp).getCtMember();
	        return match(cm);
	    } else {
	        return false;	        
	    }
    }
	
	boolean match(CtMember cm) {
	    boolean b1 = modifiers.match(cm.getModifiers());
	    boolean b2 = declaredClass.match(cm.getDeclaringClass());
	    return b1 && b2;
	}
}
