/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPMethod;
import gluonj.ast.joinpoint.JoinPoint;
import gluonj.ast.pattern.IdPattern;
import javassist.CtBehavior;
import javassist.CtMember;
import javassist.CtMethod;
import javassist.NotFoundException;

/**
 *  
 */
public class PcsMethod extends PcsBehavior {

    PcsMethod() {
        super();
    }

    public PcsMethod(PcsModifier mod, PcsType returntype,
            PcsType declaredclass, IdPattern n, PcsParameter[] param,
            PcsType[] throwable) {

        super(mod, declaredclass, param, throwable);
        returnType = returntype;
        name = n;
    }

    PcsType returnType;

    IdPattern name;

    public void accept(PcsVisitor v) {
        v.atMethodPattern(this);
    }

    public IdPattern getName() {
        return name;
    }

    public PcsType getReturnType() {
        return returnType;
    }

    public String toString() {
        return modifiers + " " + returnType + " " + declaredClass + " " + name
                + " (" + listToString(parameter) + ") throws "
                + listToString(throwable);

    }

    public boolean match(JoinPoint jp) {

        if (!super.match(jp)) {
            return false;
        }

        if (jp.getJPType() == JoinPoint.METHOD) {
            return match(((JPMethod) jp).getCtMethod());
        } else {
            return false;
        }
    }

    boolean match(CtMethod cm) {
        try {
            boolean b1 = returnType.match(cm.getReturnType());
            boolean b2 = name.match(cm.getName());
            return b1 && b2;
        } catch (NotFoundException e) {
            onNotFound(e);
            return false;
        }
    }

    boolean match(CtMember cm) {
        if (super.match(cm)) {
            if (cm instanceof CtMethod) {
                return match((CtMethod) cm);
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    boolean match(CtBehavior cb) {
        if (super.match(cb)) {
            if (cb instanceof CtMethod) {
                return match((CtMethod) cb);
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
}
