/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPClass;
import gluonj.ast.joinpoint.JoinPoint;
import gluonj.ast.pattern.ArrayTypeNamePattern;
import gluonj.ast.pattern.DotDotTypeNamePattern;
import gluonj.ast.pattern.DotTypeNamePattern;
import gluonj.ast.pattern.IdPattern;
import gluonj.ast.pattern.IdentifierTypeNamePattern;
import gluonj.ast.pattern.PrimitiveTypeNamePattern;
import gluonj.ast.pattern.SingleTypeNamePattern;
import gluonj.ast.pattern.SubTypeNamePattern;
import gluonj.ast.pattern.TypeNamePattern;
import gluonj.ast.pattern.TypeNamePatternVisitor;
import gluonj.weaver.InternalError;
import javassist.CtClass;

/*
 * 
 * |CgJbgŌ^w肷p^[\ "int", "java.lang.String", "mypackage.*",
 * "usr..*.test.*", "..", "*", "Visitor+", (String || Integer)
 * (usr.compiler.ast.Visitor+ && Test*)
 */

public class PcsType extends PointcutShadow {

    public static final PcsType ANY_TYPE = new PcsType(
            new PcsModifier(), new IdentifierTypeNamePattern(new IdPattern(
                    "*")));

    PcsModifier modifier;

    TypeNamePattern name;

    public PcsType(PcsModifier m, TypeNamePattern n) {
        modifier = m;
        name = n;
    }

    PcsType() {
    }

    public PcsModifier getModifier() {
        return modifier;
    }

    public void accept(PcsVisitor v) {
        v.atTypePattern(this);
    }

    public TypeNamePattern getTypeNamePattern() {
        return name;
    }
    

    public boolean specifySingleType() {
        return true;
    }
    
    /*
     * ̃p^[w肷NXo ̃NXw肷ꍇ RuntimeException
     */
    public String getSoleClassName() {
        final StringBuffer ret = new StringBuffer();

        name.accept(new TypeNamePatternVisitor() {
            public void atArrayTypeName(ArrayTypeNamePattern pattern) {
                ret.append(pattern.getSoleClassName());
            }

            public void atSubTypeName(SubTypeNamePattern pattern) {
                throw new RuntimeException(
                        "this TypePattern specify more than two classes, not only one class.");
            }

            public void atPrimitiveTypeName(PrimitiveTypeNamePattern pattern) {
                ret.append(pattern.getSoleClassName());
            }

            public void atDotDotTypeName(DotDotTypeNamePattern pattern) {
                ret.append(pattern.getSoleClassName());
            }

            public void atDotTypeName(DotTypeNamePattern pattern) {
                ret.append(pattern.getSoleClassName());
            }

            public void atIdentifierTypeName(IdentifierTypeNamePattern pattern) {
                ret.append(pattern.getSoleClassName());
            }
            public void atSingleTypeNamePatttern(SingleTypeNamePattern pattern) {
                ret.append(pattern.getSoleClassName());
            }
        });
        if (!ret.toString().equals("")) {
            return ret.toString();
        }
        throw new InternalError(
                "this TypePattern specify more than two classes, not only one class.");
    }

    public String toString() {
        return modifier + " " + name;
    }
    public boolean match(JoinPoint jp) {
        if(jp.getJPType() == JoinPoint.CLASS) {
            return match(((JPClass) jp).getCtClass());
        } else {
            return false;    
        }
    }

    public boolean match(CtClass declaringClass) {
        boolean b1 = modifier.match(declaringClass.getModifiers());
        boolean b2 = name.match(declaringClass.getName());
        return b1 && b2;
    }
}