/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pcs;

import gluonj.ast.joinpoint.JPFactory;
import gluonj.ast.joinpoint.JoinPoint;
import gluonj.weaver.InternalError;
import gluonj.weaver.UnSupportedException;

import java.lang.reflect.Modifier;

import javassist.CannotCompileException;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.expr.Cast;
import javassist.expr.ExprEditor;
import javassist.expr.FieldAccess;
import javassist.expr.Handler;
import javassist.expr.Instanceof;
import javassist.expr.MethodCall;
import javassist.expr.NewArray;
import javassist.expr.NewExpr;

public class PcsTypeDetail extends PcsType {

    private PointcutShadow pcs;

    public PcsTypeDetail(PointcutShadow p) {
        pcs = p;
    }

    public boolean specifySingleType() {
        return false;
    }
    public String getSoleClassName() {
        throw new InternalError("This pointcut shadow specifies more than two types.");
    }
    
    public boolean match(CtClass cc) {

        CtMethod[] cm = cc.getDeclaredMethods();
        CtConstructor[] ccn = cc.getDeclaredConstructors();

        for (int i = 0; i < cm.length; i++) {
            if (!Modifier.isAbstract(cm[i].getModifiers())) {
                JoinPoint jp = JPFactory.makeJP(cm[i]);
                if (pcs.match(jp)) {
                    return true;
                }
            }
        }
        for (int i = 0; i < ccn.length; i++) {
            JoinPoint jp = JPFactory.makeJP(ccn[i]);
            if (pcs.match(jp)) {
                return true;
            }
        }
        for (int i = 0; i < cm.length; i++) {
            if (!Modifier.isAbstract(cm[i].getModifiers())) {
                ExprMatcher matcher = new ExprMatcher();
                try {
                    cm[i].instrument(matcher);
                } catch (CannotCompileException e) {
                    throw new InternalError(e);
                }
                if(matcher.result) {
                    return true;
                }
            }
        }
        for (int i = 0; i < ccn.length; i++) {
            ExprMatcher matcher = new ExprMatcher();
            try {
                cm[i].instrument(matcher);
            } catch (CannotCompileException e) {
                throw new InternalError(e);
            }
            if(matcher.result) {
                return true;
            }
        }
        return false;
    }

    private class ExprMatcher extends ExprEditor {

        private boolean result = false;

        public void edit(Cast arg0) throws CannotCompileException {
            throw new UnSupportedException("not implemented.");
        }

        public void edit(FieldAccess arg0) throws CannotCompileException {
            if (!result) {
                result = pcs.match(JPFactory.makeJP(arg0));
            }
        }

        public void edit(Handler arg0) throws CannotCompileException {
            if (!result) {
                result = pcs.match(JPFactory.makeJP(arg0));
            }
        }

        public void edit(Instanceof arg0) throws CannotCompileException {
            throw new UnSupportedException("not implemented.");
        }

        public void edit(MethodCall arg0) throws CannotCompileException {
            if (!result) {
                result = pcs.match(JPFactory.makeJP(arg0));
            }
        }

        public void edit(NewArray arg0) throws CannotCompileException {
            throw new UnSupportedException("not implemented.");
        }

        public void edit(NewExpr arg0) throws CannotCompileException {
            throw new UnSupportedException("not implemented.");
        }
    }
}