/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
/*
 * Created on 2005/06/10
 */
package gluonj.ast.pointcut;

import gluonj.ast.joinpoint.JoinPoint;
import gluonj.ast.pcs.PointcutResidue;
import gluonj.ast.pcs.PointcutShadow;
import gluonj.ast.pointcut.visitor.Visitor;
import gluonj.weaver.JoinPointConstants;

/**
 * @author rei
 */
public class ASTPCShadow extends ASTPointcutPrimitive {

    private PointcutShadow shadow;

    private PointcutResidue residue;

    public ASTPCShadow(PointcutShadow s, PointcutResidue r) {
        shadow = s;
        residue = r;
    }

    public PointcutResidue match(JoinPoint jp) {
        if (shadow.match(jp)) {
            return residue;
        } else {
            return PointcutResidue.FALSE;
        }
    }
    
    public PointcutShadow getShadow() {
        return shadow;
    }
    public PointcutResidue getResidue() {
        return residue;
    }

    public void accept(Visitor v) {
        v.atASTCompieldPC(this);
    }

    public int getType() {
        return JoinPointConstants.SHADOW;
    }

    public String toString() {
        return "shadow(" + shadow + "), residue(" + residue + ")";
    }
}
