/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pointcut;

import gluonj.weaver.JoinPointConstants;

import java.util.Iterator;
import java.util.List;

public class PointcutExprUtil implements JoinPointConstants {

    /**
     * 
     * @param pcList -
     *            List <List <ASTPointcutPrimitive>>
     * @return Abstruct syntax tree which form is disjunctive normal form.
     */
    public static ASTPointcutExpr makeDisjunctive(List pcList) {

        int size = pcList.size();

        if (size == 0) {
            return null;
        }

        if (size == 1) {
            return makeConjunctive((List) pcList.get(0));

        } else {
            Iterator iter = pcList.iterator();
            ASTOr root = new ASTOr();
            root.setLeft(makeConjunctive((List) iter.next()));
            ASTOr current = root;

            for (int i = 1; i < size - 1; i++) {
                ASTOr new_or = new ASTOr();
                new_or.setLeft(makeConjunctive((List) iter.next()));
                current.setRight(new_or);
                current = new_or;
            }
            current.setRight(makeConjunctive((List) iter.next()));
            return root;
        }
    }
    
    
    /**
     * @param pcList -
     *            List <ASTPointcutPrimitive>
     */
    public static ASTPointcutExpr makeConjunctive(List pcList) {

        int size = pcList.size();

        if (size == 1) {
            return (ASTPointcutExpr) pcList.get(0);

        } else {
            Iterator iter = pcList.iterator();
            ASTAnd root = new ASTAnd();
            root.setLeft((ASTPointcutExpr) iter.next());
            ASTAnd current = root;

            for (int i = 1; i < size - 1; i++) {
                ASTAnd new_and = new ASTAnd();
                new_and.setLeft((ASTPointcutExpr) iter.next());
                current.setRight(new_and);
                current = new_and;
            }
            current.setRight((ASTPointcutExpr) iter.next());
            return root;
        }
    }

    public static ASTPointcutExpr getNode(ASTPointcutExpr e, String pattern) {

        if (pattern.equals("")) {
            return e;
        } else if (pattern.startsWith("L")) {
            return getNode(((ASTBinaryExpr) e).getLeft(), pattern.substring(1));
        } else if (pattern.startsWith("R")) {
            return getNode(((ASTBinaryExpr) e).getRight(), pattern.substring(1));
        } else if (pattern.startsWith("D")) {
            return ((ASTNot) e).getChild();
        }
        throw new NullPointerException(
                "The pattern must be consist of only 3 characters. L, R and D. ");
    }

    static public boolean isShadowSpecifier(int type) {
        return (type > 0 && type <= 20) || type == SHADOW;
    }

    static public boolean isBehaviorStatementSpecifier(int type) {
        return type == CALL || type == FIELD_GET || type == FIELD_SET;
    }

    static public boolean isWithin(int type) {
        return type == WITHIN || type == WITHINCODE;
    }

    static public boolean isResidueSpecifier(int type) {
        return (type > 30 && type <= 40) || type == RESIDUE;
    }

    /*
     * static public ASTPointcutPrimitive and( ASTPointcutPrimitive p1,
     * ASTPointcutPrimitive p2) {
     * 
     * return null;
     *  }
     * 
     * static private ASTFieldGet andFieldGet(ASTFieldGet p1, ASTFieldGet p2) {
     * 
     * GetPattern f = andGetPattern(p1.getPattern(), p2.getPattern()); if (f ==
     * null) { return null; } else { return new ASTFieldGet(f); } }
     * 
     * private static GetPattern andGetPattern( GetPattern pattern, GetPattern
     * pattern2) {
     * 
     * TypePattern d = and(pattern.getDeclaredClass(),
     * pattern2.getDeclaredClass()); BehaviorPattern b = and(pattern.getWhere(),
     * pattern2.getWhere()); FieldPattern f = and(pattern.getField(),
     * pattern2.getField()); if (f == null || b == null || d == null) { return
     * null; } else { GetPattern p = new GetPattern(d, b, f); return p; } }
     * 
     * private static SetPattern andSetPattern( SetPattern pattern, SetPattern
     * pattern2) {
     * 
     * TypePattern d = and(pattern.getDeclaredClass(),
     * pattern2.getDeclaredClass()); BehaviorPattern b = and(pattern.getWhere(),
     * pattern2.getWhere()); FieldPattern f = and(pattern.getField(),
     * pattern2.getField()); if (f == null || b == null || d == null) { return
     * null; } else { SetPattern p = new SetPattern(d, b, f); return p; } }
     * 
     * private static CallPattern andCallPattern( CallPattern pattern,
     * CallPattern pattern2) {
     * 
     * if (pattern instanceof MethodCallPattern && pattern2 instanceof
     * MethodCallPattern) {
     * 
     * MethodCallPattern p1 = (MethodCallPattern) pattern; MethodCallPattern p2 =
     * (MethodCallPattern) pattern2; return andMethodCallPattern(p1, p2);
     *  } else if ( pattern instanceof NewExprPattern && pattern2 instanceof
     * NewExprPattern) {
     * 
     * NewExprPattern p1 = (NewExprPattern) pattern; NewExprPattern p2 =
     * (NewExprPattern) pattern2; return andNewExprPattern(p1, p2);
     *  } else { return null; } }
     * 
     * private static NewExprPattern andNewExprPattern( NewExprPattern pattern,
     * NewExprPattern pattern2) {
     * 
     * TypePattern d = and(pattern.getDeclaredClass(),
     * pattern2.getDeclaredClass()); BehaviorPattern b = and(pattern.getWhere(),
     * pattern2.getWhere()); ConstructorPattern c = and(pattern.getNewExpr(),
     * pattern2.getNewExpr()); if (c == null || b == null || d == null) { return
     * null; } else { return new NewExprPattern(d, b, c); } }
     * 
     * private static MethodCallPattern andMethodCallPattern( MethodCallPattern
     * pattern, MethodCallPattern pattern2) {
     * 
     * TypePattern d = and(pattern.getDeclaredClass(),
     * pattern2.getDeclaredClass()); BehaviorPattern b = and(pattern.getWhere(),
     * pattern2.getWhere()); MethodPattern c = and(pattern.getMethod(),
     * pattern2.getMethod()); if (c == null || b == null || d == null) { return
     * null; } else { return new MethodCallPattern(d, b, c); } }
     * 
     * private static BehaviorPattern and( BehaviorPattern pattern,
     * BehaviorPattern pattern2) {
     * 
     * if (pattern instanceof MethodPattern && pattern2 instanceof
     * MethodPattern) { return and((MethodPattern) pattern, (MethodPattern)
     * pattern2); } else if ( pattern instanceof ConstructorPattern && pattern2
     * instanceof ConstructorPattern) { return and((ConstructorPattern) pattern,
     * (ConstructorPattern) pattern2); } else { return null; } }
     * 
     * private static ConstructorPattern and( ConstructorPattern pattern,
     * ConstructorPattern pattern2) { return null; }
     * 
     * private static MethodPattern and( MethodPattern pattern, MethodPattern
     * pattern2) { return null; }
     * 
     * private static TypePattern and(TypePattern pattern, TypePattern pattern2) {
     * return null; }
     * 
     * private static FieldPattern and( FieldPattern pattern, FieldPattern
     * pattern2) { return null; } /* private static class AndVisitor extends
     * Visitor {
     * 
     * private ASTPointcutPrimitive expr2;
     * 
     * private ASTPointcutPrimitive result;
     * 
     * AndVisitor(ASTPointcutPrimitive e) { expr2 = e; result = null; }
     * 
     * public void atASTFieldGet(final ASTFieldGet expr) { expr2.accept(new
     * Visitor() { public void atASTFieldGet(ASTFieldGet expr2) { result =
     * andFieldGet(expr, expr2); }
     * 
     * public void atASTWithin(ASTWithin expr2) {
     * 
     * BehaviorPattern old_b = expr.getPattern().getWhere(); TypePattern
     * new_declaredclass = and(expr2.getPattern(), old_b.getDeclaredClass());
     * BehaviorPattern new_where = null;
     * 
     * if (old_b instanceof MethodPattern) { new_where = new MethodPattern(
     * old_b.getModifier(), ((MethodPattern) old_b).getReturnType(),
     * new_declaredclass, ((MethodPattern) old_b).getName(),
     * old_b.getParameter(), old_b.getThrowable()); } else if (old_b instanceof
     * ConstructorPattern) { new_where = new ConstructorPattern(
     * old_b.getModifier(), new_declaredclass, old_b.getParameter(),
     * old_b.getThrowable()); } result = new ASTFieldGet(null); } }); }
     * 
     * public void atASTFieldSet(final ASTFieldSet expr) { expr2.accept(new
     * Visitor() { public void atASTFieldSet(ASTFieldSet expr2) { result =
     * and(expr, expr2); } }); }
     * 
     * public void atASTMethodCall(final ASTMethodCall expr) { expr2.accept(new
     * Visitor() { public void atASTMethodCall(ASTMethodCall expr2) { result =
     * and(expr, expr2); } }); }
     * 
     * public void atASTMethodExecution(final ASTMethodExecution expr) {
     * expr2.accept(new Visitor() { public void
     * atASTMethodExcecution(ASTMethodExecution expr2) { result = and(expr,
     * expr2); } }); }
     *  }
     * 
     * static private class AndPatternVisitor extends PatternVisitor {
     * 
     * Pattern p1;
     * 
     * CallPattern newCallPattern;
     * 
     * AndPatternVisitor(Pattern p) { p1 = p; }
     * 
     * public CallPattern getNewCallPattern() { return newCallPattern; }
     * 
     * public void atMethodCallPattern(final MethodCallPattern p2) {
     * p1.accept(new PatternVisitor() { public void
     * atMethodCallPattern(MethodCallPattern p1) { newCallPattern =
     * andCallPattern(p1, p2); } }); }
     * 
     * public void atNewExprPattern(final NewExprPattern p2) { p1.accept(new
     * PatternVisitor() { public void atNewExprPattern(NewExprPattern p1) {
     * newCallPattern = andNewExprPattern(p1, p2); } }); } }
     */
}
