/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pointcut.visitor;

import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;

import java.util.LinkedList;
import java.util.List;

/**
 * You can use this translator ONLY with Traverser.traverseAndOrNotPrimitive().
 * 
 */
public abstract class PcPrimitiveTranslator extends Visitor {

    private List stack = new LinkedList();
    
    public abstract ASTPointcutExpr changePcPrimitive(
            ASTPointcutPrimitive a);

    public final void beforeASTAnd(ASTAnd expr) {
        
    }
    public final void beforeASTOr(ASTOr expr) {
        
    }
    public final void beforeASTNot(ASTNot expr) {
        
    }
    public final void afterASTAnd(ASTAnd expr) {
        ASTPointcutExpr right = pop();
        ASTPointcutExpr left = pop();
        push(new ASTAnd(left, right));
    }

    public final void afterASTOr(ASTOr expr) {
        ASTPointcutExpr right = pop();
        ASTPointcutExpr left = pop();
        push(new ASTOr(left, right));
    }

    public final void afterASTNot(ASTNot expr) {
        ASTPointcutExpr child = pop();
        push(new ASTNot(child));
    }
    public final void atAstPointcutPrimitive(ASTPointcutPrimitive expr) {
        ASTPointcutExpr e = changePcPrimitive(expr);
        push(e);
    }
    
    private void push(ASTPointcutExpr e) {
        stack.add(0, e);
    }
    private ASTPointcutExpr pop() {
        return (ASTPointcutExpr) stack.remove(0);
    }
    
    public ASTPointcutExpr getNewPC() {
        return pop();
    }
}
