/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pointcut.visitor;

import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.ASTUnaryExpr;
import gluonj.weaver.InternalError;
import gluonj.weaver.JoinPointConstants;

import java.util.LinkedList;
import java.util.List;

public class Traverser {

    static public void traverseAndOrUnary(ASTPointcutExpr e, Visitor v) {

        if (e.getType() == JoinPointConstants.AND) {
            ASTAnd a = (ASTAnd) e;
            v.beforeASTAnd(a);
            traverseAndOrUnary(a.getLeft(), v);
            traverseAndOrUnary(a.getRight(), v);
            v.afterASTAnd(a);
        } else if (e.getType() == JoinPointConstants.OR) {
            ASTAnd a = (ASTAnd) e;
            v.beforeASTAnd(a);
            traverseAndOrUnary(a.getLeft(), v);
            traverseAndOrUnary(a.getRight(), v);
            v.afterASTAnd(a);
        } else {
            ASTUnaryExpr a = (ASTUnaryExpr) e;
            v.atAstUnaryExpr(a);
        }
    }

    static public void traverseAndOrNotPrimitive(ASTPointcutExpr e, Visitor v) {

        if (e.getType() == JoinPointConstants.AND) {
            ASTAnd a = (ASTAnd) e;
            v.beforeASTAnd(a);
            traverseAndOrNotPrimitive(a.getLeft(), v);
            traverseAndOrNotPrimitive(a.getRight(), v);
            v.afterASTAnd(a);
        } else if (e.getType() == JoinPointConstants.OR) {
            ASTOr a = (ASTOr) e;
            v.beforeASTOr(a);
            traverseAndOrNotPrimitive(a.getLeft(), v);
            traverseAndOrNotPrimitive(a.getRight(), v);
            v.afterASTOr(a);
        } else if (e.getType() == JoinPointConstants.NOT) {
            ASTNot a = (ASTNot) e;
            v.beforeASTNot(a);
            traverseAndOrNotPrimitive(a.getChild(), v);
            v.afterASTNot(a);
        } else {
            ASTPointcutPrimitive a = (ASTPointcutPrimitive) e;
            v.atAstPointcutPrimitive(a);
        }
    }

    static public void traverseDNF(ASTPointcutExpr e, DNFVisitor v) {
        traverseDNF0(e, v);
        v.afterDisjunctive();
    }

    static public void traverseDNF0(ASTPointcutExpr e, DNFVisitor v) {

        if (e.getType() == JoinPointConstants.OR) {

            ASTOr or = (ASTOr) e;
            traverseDNF0(or.getLeft(), v);
            traverseDNF0(or.getRight(), v);

        } else if (e.getType() == JoinPointConstants.AND) {
            ASTAnd and = (ASTAnd) e;
            List elements = new LinkedList();
            traverseCNF(and.getLeft(), v, elements);
            traverseCNF(and.getRight(), v, elements);
            v.atConjunctive(elements);

        } else {
            List elements = new LinkedList();
            elements.add(e);
            v.atConjunctive(elements);
        }
    }

    static private void traverseCNF(ASTPointcutExpr e, DNFVisitor v,
            List elements) {

        if (e.getType() == JoinPointConstants.OR) {
            throw new InternalError("The form of parameter e is not DNF.");
        } else if (e.getType() == JoinPointConstants.AND) {
            ASTAnd and = (ASTAnd) e;
            traverseCNF(and.getLeft(), v, elements);
            traverseCNF(and.getRight(), v, elements);
        } else {
            elements.add(e);
        }
    }
}