/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pointcut.visitor;

import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTArgs;
import gluonj.ast.pointcut.ASTBinaryExpr;
import gluonj.ast.pointcut.ASTCflow;
import gluonj.ast.pointcut.ASTCflowBelow;
import gluonj.ast.pointcut.ASTFieldGet;
import gluonj.ast.pointcut.ASTFieldSet;
import gluonj.ast.pointcut.ASTMethodCall;
import gluonj.ast.pointcut.ASTMethodExecution;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPCReference;
import gluonj.ast.pointcut.ASTPCShadow;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.ASTResidue;
import gluonj.ast.pointcut.ASTShadow;
import gluonj.ast.pointcut.ASTTarget;
import gluonj.ast.pointcut.ASTThis;
import gluonj.ast.pointcut.ASTUnaryExpr;
import gluonj.ast.pointcut.ASTWithin;
import gluonj.ast.pointcut.ASTWithincode;

public abstract class Visitor {

    // depth 1

    public void atAstPointcutExpr(ASTPointcutExpr expr) {

    }

    // depth 2

    public void afterAstBinaryExpr(ASTBinaryExpr expr) {

    }

    public void beforeAstBinaryExpr(ASTBinaryExpr expr) {

    }

    public void atAstUnaryExpr(ASTUnaryExpr expr) {

    }

    // depth 3

    public void afterASTOr(ASTOr expr) {

    }

    public void afterASTAnd(ASTAnd expr) {

    }

    public void afterASTNot(ASTNot expr) {

    }

    public void beforeASTOr(ASTOr expr) {

    }

    public void beforeASTAnd(ASTAnd expr) {

    }

    public void beforeASTNot(ASTNot expr) {

    }

    public void atAstPointcutPrimitive(ASTPointcutPrimitive expr){

    }

    // depth 4

    public void atASTWithin(ASTWithin expr) {

    }

    public void atASTMethodExecution(ASTMethodExecution expr) {

    }

    public void atASTMethodCall(ASTMethodCall expr) {

    }

    public void atASTFieldGet(ASTFieldGet expr) {

    }

    public void atASTFieldSet(ASTFieldSet expr) {

    }

    public void atASTTarget(ASTTarget expr) {

    }

    public void atASTThis(ASTThis expr) {

    }

    public void atASTArgs(ASTArgs expr) {

    }

    public void atASTPCReference(ASTPCReference expr){}

    public void atASTCflow(ASTCflow expr){}

    public void atASTResidue(ASTResidue residue){}

    public void atASTWithincode(ASTWithincode expr){}

    public void atASTShadow(ASTShadow shadow){}

    /**
     * @param compiledPC
     */
    public void atASTCompieldPC(ASTPCShadow compiledPC){}

    public void atASTCflowBelow(ASTCflowBelow expr) {
    }

}
