/*
 * The Relaxer artifact
 * Copyright (c) 2000-2003, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package gluonj.ast.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * <b>ElmAdvice</b> is generated from gluonj.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="behavior">
 *   <zeroOrMore>
 *     <ref name="param"/>
 *   </zeroOrMore>
 *   <ref name="pointcut"/>
 *   <choice>
 *     <element name="before">
 *       <text/>
 *     </element>
 *     <element name="after">
 *       <text/>
 *     </element>
 *     <element name="around">
 *       <text/>
 *     </element>
 *   </choice>
 * </element>
 * -->
 * <!-- for javadoc -->
 * <pre> &lt;element name="behavior"&gt;
 *   &lt;zeroOrMore&gt;
 *     &lt;ref name="param"/&gt;
 *   &lt;/zeroOrMore&gt;
 *   &lt;ref name="pointcut"/&gt;
 *   &lt;choice&gt;
 *     &lt;element name="before"&gt;
 *       &lt;text/&gt;
 *     &lt;/element&gt;
 *     &lt;element name="after"&gt;
 *       &lt;text/&gt;
 *     &lt;/element&gt;
 *     &lt;element name="around"&gt;
 *       &lt;text/&gt;
 *     &lt;/element&gt;
 *   &lt;/choice&gt;
 * &lt;/element&gt;
 * </pre>
 *
 * @version gluonj.rng (Wed Dec 07 20:20:12 JST 2005)
 * @author  Relaxer 1.0 (http://www.relaxer.org)
 */
public class ElmAdvice implements java.io.Serializable, Cloneable, IRVisitable, IRNode, IElmAspectChoice {
    // List<ElmParam>
    private java.util.List param_ = new java.util.ArrayList();
    private String pointcut_;
    private IElmAdviceChoice content_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>ElmAdvice</code>.
     *
     */
    public ElmAdvice() {
        pointcut_ = "";
    }

    /**
     * Creates a <code>ElmAdvice</code>.
     *
     * @param source
     */
    public ElmAdvice(ElmAdvice source) {
        setup(source);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public ElmAdvice(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public ElmAdvice(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>ElmAdvice</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public ElmAdvice(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ElmAdvice(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>ElmAdvice</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ElmAdvice(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ElmAdvice(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ElmAdvice(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ElmAdvice(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>ElmAdvice</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public ElmAdvice(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the ElmAdvice <code>source</code>.
     *
     * @param source
     */
    public void setup(ElmAdvice source) {
        int size;
        this.param_.clear();
        size = source.param_.size();
        for (int i = 0;i < size;i++) {
            addParam((ElmParam)source.getParam(i).clone());
        }
        setPointcut(source.getPointcut());
        if (source.content_ != null) {
            setContent((IElmAdviceChoice)source.getContent().clone());
        }
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        param_.clear();
        while (true) {
            if (ElmParam.isMatch(stack)) {
                addParam(new ElmParam(stack));
            } else {
                break;
            }
        }
        pointcut_ = URelaxer.getElementPropertyAsString(stack.popElement());
        if (ElmAdviceBefore.isMatch(stack)) {
            setContent(new ElmAdviceBefore(stack));
        } else if (ElmAdviceAfter.isMatch(stack)) {
            setContent(new ElmAdviceAfter(stack));
        } else if (ElmAdviceAround.isMatch(stack)) {
            setContent(new ElmAdviceAround(stack));
        } else {
            throw (new IllegalArgumentException());
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new ElmAdvice(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("behavior");
        int size;
        size = this.param_.size();
        for (int i = 0;i < size;i++) {
            ElmParam value = (ElmParam)this.param_.get(i);
            value.makeElement(element);
        }
        URelaxer.setElementPropertyByString(element, "pointcut", this.pointcut_);
        this.content_.makeElement(element);
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>ElmAdvice</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>ElmAdvice</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the ElmParam property <b>param</b>.
     *
     * @return ElmParam[]
     */
    public final ElmParam[] getParam() {
        ElmParam[] array = new ElmParam[param_.size()];
        return ((ElmParam[])param_.toArray(array));
    }

    /**
     * Sets the ElmParam property <b>param</b>.
     *
     * @param param
     */
    public final void setParam(ElmParam[] param) {
        this.param_.clear();
        for (int i = 0;i < param.length;i++) {
            addParam(param[i]);
        }
        for (int i = 0;i < param.length;i++) {
            param[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the ElmParam property <b>param</b>.
     *
     * @param param
     */
    public final void setParam(ElmParam param) {
        this.param_.clear();
        addParam(param);
        if (param != null) {
            param.rSetParentRNode(this);
        }
    }

    /**
     * Adds the ElmParam property <b>param</b>.
     *
     * @param param
     */
    public final void addParam(ElmParam param) {
        this.param_.add(param);
        if (param != null) {
            param.rSetParentRNode(this);
        }
    }

    /**
     * Adds the ElmParam property <b>param</b>.
     *
     * @param param
     */
    public final void addParam(ElmParam[] param) {
        for (int i = 0;i < param.length;i++) {
            addParam(param[i]);
        }
        for (int i = 0;i < param.length;i++) {
            param[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the ElmParam property <b>param</b>.
     *
     * @return int
     */
    public final int sizeParam() {
        return (param_.size());
    }

    /**
     * Gets the ElmParam property <b>param</b> by index.
     *
     * @param index
     * @return ElmParam
     */
    public final ElmParam getParam(int index) {
        return ((ElmParam)param_.get(index));
    }

    /**
     * Sets the ElmParam property <b>param</b> by index.
     *
     * @param index
     * @param param
     */
    public final void setParam(int index, ElmParam param) {
        this.param_.set(index, param);
        if (param != null) {
            param.rSetParentRNode(this);
        }
    }

    /**
     * Adds the ElmParam property <b>param</b> by index.
     *
     * @param index
     * @param param
     */
    public final void addParam(int index, ElmParam param) {
        this.param_.add(index, param);
        if (param != null) {
            param.rSetParentRNode(this);
        }
    }

    /**
     * Remove the ElmParam property <b>param</b> by index.
     *
     * @param index
     */
    public final void removeParam(int index) {
        this.param_.remove(index);
    }

    /**
     * Remove the ElmParam property <b>param</b> by object.
     *
     * @param param
     */
    public final void removeParam(ElmParam param) {
        this.param_.remove(param);
    }

    /**
     * Clear the ElmParam property <b>param</b>.
     *
     */
    public final void clearParam() {
        this.param_.clear();
    }

    /**
     * Gets the String property <b>pointcut</b>.
     *
     * @return String
     */
    public final String getPointcut() {
        return (pointcut_);
    }

    /**
     * Sets the String property <b>pointcut</b>.
     *
     * @param pointcut
     */
    public final void setPointcut(String pointcut) {
        this.pointcut_ = pointcut;
    }

    /**
     * Gets the IElmAdviceChoice property <b>content</b>.
     *
     * @return IElmAdviceChoice
     */
    public final IElmAdviceChoice getContent() {
        return (content_);
    }

    /**
     * Sets the IElmAdviceChoice property <b>content</b>.
     *
     * @param content
     */
    public final void setContent(IElmAdviceChoice content) {
        this.content_ = content;
        if (content != null) {
            content.rSetParentRNode(this);
        }
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<behavior");
        content_.makeTextAttribute(buffer);
        buffer.append(">");
        size = this.param_.size();
        for (int i = 0;i < size;i++) {
            ElmParam value = (ElmParam)this.param_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("<pointcut>");
        buffer.append(URelaxer.escapeCharData(URelaxer.getString(getPointcut())));
        buffer.append("</pointcut>");
        content_.makeTextElement(buffer);
        buffer.append("</behavior>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<behavior");
        content_.makeTextAttribute(buffer);
        buffer.write(">");
        size = this.param_.size();
        for (int i = 0;i < size;i++) {
            ElmParam value = (ElmParam)this.param_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("<pointcut>");
        buffer.write(URelaxer.escapeCharData(URelaxer.getString(getPointcut())));
        buffer.write("</pointcut>");
        content_.makeTextElement(buffer);
        buffer.write("</behavior>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<behavior");
        content_.makeTextAttribute(buffer);
        buffer.print(">");
        size = this.param_.size();
        for (int i = 0;i < size;i++) {
            ElmParam value = (ElmParam)this.param_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("<pointcut>");
        buffer.print(URelaxer.escapeCharData(URelaxer.getString(getPointcut())));
        buffer.print("</pointcut>");
        content_.makeTextElement(buffer);
        buffer.print("</behavior>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPointcutAsString() {
        return (URelaxer.getString(getPointcut()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPointcutByString(String string) {
        setPointcut(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Accepts the Visitor for enter behavior.
     *
     * @param visitor
     * @return boolean
     */
    public boolean enter(IRVisitor visitor) {
        return (visitor.enter(this));
    }

    /**
     * Accepts the Visitor for leave behavior.
     *
     * @param visitor
     */
    public void leave(IRVisitor visitor) {
        visitor.leave(this);
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public final IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public final void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(param_);
        if (content_ != null) {
            classNodes.add(content_);
        }
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>ElmAdvice</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "behavior")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        while (true) {
            if (!ElmParam.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        child = target.popElement();
        if (child == null) {
            return (false);
        }
        if (!URelaxer.isTargetElement(child, "pointcut")) {
            return (false);
        }
        $match$ = true;
        if (ElmAdviceBefore.isMatchHungry(target)) {
            $match$ = true;
        } else if (ElmAdviceAfter.isMatchHungry(target)) {
            $match$ = true;
        } else if (ElmAdviceAround.isMatchHungry(target)) {
            $match$ = true;
        } else {
            return (false);
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>ElmAdvice</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>ElmAdvice</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
