/*
 * The Relaxer artifact
 * Copyright (c) 2000-2003, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package gluonj.ast.xml;

/**
 * @version gluonj.rng 1.0 (Wed Dec 07 20:20:13 JST 2005)
 * @author  Relaxer 1.0 (http://www.relaxer.org)
 */
public interface IRVisitor {
    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmAspect visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmAspect visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmPointcutDecl visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmPointcutDecl visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmParam visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmParam visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmAdvice visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmAdvice visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmAdviceBefore visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmAdviceBefore visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmAdviceAfter visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmAdviceAfter visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmAdviceAround visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmAdviceAround visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmImport visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmImport visitable);

    /**
     * Visits this node for enter behavior.
     *
     * @param visitable
     * @return boolean
     */
    boolean enter(ElmInjection visitable);

    /**
     * Visits this node for leave behavior.
     *
     * @param visitable
     */
    void leave(ElmInjection visitable);
}
