/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.parser;

import java.lang.reflect.Field;
import java.util.HashMap;

public class Lexer implements TokenId {

    Lex lex;

    public Lexer(String s) {
        lex = new Lex(s);
        str = s;
    }

    public int get() {
        return lex.get();
    }

    /*
     * lookAhead(0) Ŏɓǂݍޕ
     */
    public int lookAhead() {
        return lex.lookAhead(0);
    }

    public int lookAhead(int i) {
        return lex.lookAhead(i);
    }

    public String getIdentifier() {
        return lex.getString();
    }

    public String getString() {
        return lex.getString();
    }

    public long getLong() {
        return lex.getLong();
    }

    public double getDouble() {
        return lex.getDouble();
    }

    public boolean isPrimitiveType() {

        int c = lex.lookAhead(0);
        return c == BOOLEAN || c == BYTE || c == CHAR || c == DOUBLE
                || c == FLOAT || c == INT || c == LONG || c == SHORT
                || c == VOID;
    }

    public String getPrimitiveType() throws ParseException {
        String s = (String) table.get(new Integer(lex.lookAhead(0)));
        if (s == null)
            throw new ParseException("this is not primitive : "
                    + lex.lookAhead());
        return s;
    }

    static HashMap table = new HashMap();

    static {
        table.put(new Integer(BOOLEAN), "boolean");
        table.put(new Integer(BYTE), "byte");
        table.put(new Integer(CHAR), "char");
        table.put(new Integer(DOUBLE), "double");
        table.put(new Integer(FLOAT), "float");
        table.put(new Integer(INT), "int");
        table.put(new Integer(LONG), "long");
        table.put(new Integer(SHORT), "short");
        table.put(new Integer(VOID), "void");
    }

    // ad-hoc implementation for parsing a initializer of the injection code.
    private String str;

    private int markStart;

    public void startMark() {
        markStart = lex.getPosision();
    }

    public String endMark() {
        return str.substring(markStart, lex.getPosision() - 1);
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < 10; i++) {
            int c = lookAhead(i);
            if (c == Identifier) {
                buf.append(getIdentifier() + " ");
            } else if (c > 255) {
                try {
                    Class tid = Class.forName("gluonj.parser.TokenId");
                    Field[] f = tid.getFields();

                    for (int j = 0; j < f.length; j++) {
                        if (f[j].getType().equals(Integer.TYPE)
                                && f[j].getInt(null) == c) {
                            buf.append(f[j].getName());
                            break;
                        }
                    }
                } catch (Exception e) {
                    return buf.toString() + " errored " + e;
                }
            } else {
                buf.append((char) c);
            }
        }
        return buf.toString();
    }
}
