/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.runtime;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class ReflectionClosure extends Closure {

    Class thisClass;

    String methodName;

    Class[] parameterTypes;

    public ReflectionClosure(Class cn, String mn, Class[] pn) {
        thisClass = cn;
        methodName = mn;
        parameterTypes = pn;
    }

    public ReflectionClosure(Class cn, Class[] pn) {
        thisClass = cn;
        parameterTypes = pn;
    }

    public Object proceed(Object t, Object[] arg) throws Throwable {

        try {
            if (methodName != null) {

                Method[] m = thisClass.getMethods();

                Method target = thisClass.getMethod(methodName, parameterTypes);
                return target.invoke(t, arg);
            } else {
                Constructor target = thisClass.getConstructor(parameterTypes);
                return target.newInstance(arg);
            }
        } catch (SecurityException e) {
            throw makeException(e);
        } catch (NoSuchMethodException e) {
            throw makeException(e);
        } catch (IllegalArgumentException e) {
            throw makeException(e);
        } catch (IllegalAccessException e) {
            throw makeException(e);
        } catch (InvocationTargetException e) {
            throw makeException(e);
        } catch (InstantiationException e) {
            throw makeException(e);
        }
    }

    private RuntimeException makeException(Exception e) {
        return new GluonJRuntimeException("Occured when calling Closure.proceed()", e);
    }

}
