/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ui;

import gluonj.ast.Aspect;
import gluonj.parser.AspectParser;
import gluonj.weaver.AOPWeaver;
import gluonj.weaver.AOPWeaverFactory;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.Loader;
import javassist.NotFoundException;
import javassist.Translator;

public class LoadTimeWeaver {


    public static void run(String[] aspectName, String typeName, String[] args,
            ClassPool pool) throws Throwable {

        AspectParser parser = AspectParser.factory();
        Aspect[] aspects = new Aspect[aspectName.length];
        for (int i = 0; i < aspectName.length; i++) {
            aspects[i] = parser.parse(aspectName[i]);
        }
        AOPWeaver weaver = new AOPWeaverFactory().make(aspects);

        weaver.weaveIntertypeDecl(pool);

        loader = new Loader();
        try {
            loader.addTranslator(pool, new SimpleTranslator(weaver));
            loader.run(typeName, args);
        } catch (CannotCompileException e) {
            if (e instanceof AOPWeaver.CannotCompileExceptionInner) {
                AOPWeaver.CannotCompileExceptionInner ei = (AOPWeaver.CannotCompileExceptionInner) e;
                throw new gluonj.weaver.CannotCompileException(ei.getBody(), ei
                        .getCause());
            } else {
                throw new gluonj.weaver.CannotCompileException(e);
            }
        }
    }

    public static void run(String[] aspectName, String typeName, String[] args)
            throws Throwable {

        AspectParser parser = AspectParser.factory();
        Aspect[] aspects = new Aspect[aspectName.length];
        for (int i = 0; i < aspectName.length; i++) {
            aspects[i] = parser.parse(aspectName[i]);
        }
        AOPWeaver weaver = new AOPWeaverFactory().make(aspects);

        ClassPool pool = ClassPool.getDefault();
        weaver.weaveIntertypeDecl(pool);
        loader = new Loader();
        try {
            loader.addTranslator(pool, new SimpleTranslator(weaver));
            loader.run(typeName, args);
        } catch (CannotCompileException e) {
            if (e instanceof AOPWeaver.CannotCompileExceptionInner) {
                AOPWeaver.CannotCompileExceptionInner ei = (AOPWeaver.CannotCompileExceptionInner) e;
                throw new gluonj.weaver.CannotCompileException(ei.getBody(), ei
                        .getCause());
            } else {
                throw new gluonj.weaver.CannotCompileException(e);
            }
        }
    }

    public static Loader loader;


    static class SimpleTranslator implements Translator {

        private AOPWeaver weaver;

        SimpleTranslator(AOPWeaver w) {
            weaver = w;
        }

        public void onLoad(ClassPool pool, String className)
                throws NotFoundException, CannotCompileException {

            try {
                //weaver.weaveIntertypeDecl(pool.get(className), pool);
                weaver.weaveAdvice(pool.get(className), pool);
            } catch (Throwable t) {
                System.err.println(t);
                throw new CannotCompileException(t); 
            }
        }

        public void start(ClassPool arg0) throws NotFoundException,
                CannotCompileException {
        }
    }
}
