/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ui;

import java.util.HashMap;
import java.util.Map;

class OptionTable {

    Map optionMap;
    String[] arguments;
    private OptionTable(Map m, String[] args) {
        optionMap = m;
        arguments = args;
    }

    public String[] getValueOf(String opname) {
        if(optionMap.containsKey(opname)) {
            return (String[]) optionMap.get(opname);
        } else {
            return null;
        }
    }
    
    public boolean contains(String opname) {
        return optionMap.containsKey(opname);
    }

    public String[] getArgument() {
        return arguments;
    }

    public static class Factory {

        private Map opMap = new HashMap();

        public void setOptionFormat(String opname, int num) {
            opMap.put(opname, new Integer(num));
        }

        public OptionTable make(String[] args) {

            int idx = 0;
            int length = args.length;
            Map m = new HashMap();

            while (idx < length && isOp(args[idx])) {
                String op = makeOpName(args[idx++]);
                int num;
                if (opMap.containsKey(op)) {
                    num = ((Integer) opMap.get(op)).intValue();
                } else {
                    throw new RuntimeException("Option \"" + op
                            + "\" is not defined.");
                }
                String[] value = new String[num];
                for (int i = 0; i < num; i++) {
                    if (idx < length) {
                        value[i] = args[idx++];
                    } else {
                        throw new RuntimeException("Option \"" + op
                                + "\" takes " + num + " values.");
                    }
                }
                m.put(op, value);
            }
            String[] arguments;
            if (idx < length) {
                arguments = new String[length - idx];
                for (int i = idx; i < length; i++) {
                    arguments[i- idx] = args[i];
                }
            } else {
                arguments = new String[0];
            }
            return new OptionTable(m, arguments);
        }

        private boolean isOp(String str) {
            return str.startsWith("-");
        }

        private String makeOpName(String str) {
            return str.substring(1);
        }
    }
}
