/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver;

import gluonj.ast.Advice;
import gluonj.ast.Aspect;
import gluonj.ast.intertype.IntertypeDecl;
import gluonj.parser.ParseException;
import gluonj.weaver.gen.AnonymousFieldProcessor;
import gluonj.weaver.gen.CflowRemover;
import gluonj.weaver.gen.DisjunctiveNFProcessor;
import gluonj.weaver.gen.ImportApplier;
import gluonj.weaver.gen.NamedPCProcessor;
import gluonj.weaver.gen.Processor;
import gluonj.weaver.gen.ResidueProcessor;
import gluonj.weaver.gen.ShadowProcessor;
import gluonj.weaver.gen.WithinRemover;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class AOPWeaverFactory {

    public AOPWeaver make(Aspect[] aspects)
            throws gluonj.weaver.CannotCompileException, ParseException {

        aspects = compile(aspects);
        AOPWeaver weaver = new AOPWeaver();

        for (int i = 0; i < aspects.length; i++) {
            Aspect asp = aspects[i];

            for (int j = 0; j < asp.getAdvice().length; j++) {
                Advice advice = asp.getAdvice()[j];
                weaver.addAdvice(advice);
            }

            for (int j = 0; j < asp.getIntertypeDecl().length; j++) {
                IntertypeDecl decl = asp.getIntertypeDecl()[j];
                weaver.addIntertypeDecl(decl);
            }
        }
        return weaver;
    }

    private static Aspect[] compile(Aspect[] aspects)
            throws gluonj.weaver.CannotCompileException, ParseException {

        Pipe p = new Pipe().append(new CflowRemover()).append(
                new NamedPCProcessor()).append(new DisjunctiveNFProcessor())
        //                .append(new PointcutSorter())
                .append(new AnonymousFieldProcessor()).append(
                        new ResidueProcessor()).append(new WithinRemover())
                .append(new ShadowProcessor()).append(new ImportApplier());

        return p.process(aspects);
    }

    static class Pipe {

        List processorList = new LinkedList();

        public Aspect[] process(Aspect[] aspects)
                throws gluonj.weaver.CannotCompileException {
            for (Iterator iter = processorList.iterator(); iter.hasNext();) {
                Processor processor = (Processor) iter.next();
                aspects = processor.process(aspects);
            }
            return aspects;
        }

        public Pipe append(Processor processor) {
            processorList.add(processor);
            return this;
        }
    }
}