/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver;

import gluonj.ast.java.ASTConstructor;
import gluonj.ast.java.ASTMethod;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.NotFoundException;

public class BodyWeavingVisitor extends IntertypeDeclWeavingVisitor {

    BodyWeavingVisitor(ClassPool pool) {
        super(pool);
    }

    public void atASTMethod(ASTMethod method) {

        String name = method.getName();
        CtClass[] parameter = new CtClass[method.getParameter().length];
        try {
            for (int i = 0; i < parameter.length; i++) {
                parameter[i] = pool.get(method.getParameter()[i].getType()
                        .getType());
            }

            CtMethod cm = cc.getDeclaredMethod(name, parameter);
            cm.setBody("{" + method.getBody().toString() + "}");

        } catch (NotFoundException e) {
            throw new RuntimeException(
                    new gluonj.weaver.CannotCompileException(e));
        } catch (CannotCompileException e) {
            throw new RuntimeException(
                    new gluonj.weaver.CannotCompileException("Method \"" + name
                            + "\" might have an error.\n" + e.getMessage()));
        }
    }

    public void atASTConstructor(ASTConstructor constructor) {

        CtClass[] parameter = new CtClass[constructor.getParameter().length];
        try {
            for (int i = 0; i < parameter.length; i++) {
                parameter[i] = pool.get(constructor.getParameter()[i].getType()
                        .getType());
            }

            CtConstructor ccn = cc.getDeclaredConstructor(parameter);
            ccn.setBody(constructor.getBody().toString());

        } catch (NotFoundException e) {
            throw new RuntimeException(
                    new gluonj.weaver.CannotCompileException(e));
        } catch (CannotCompileException e) {
            throw new RuntimeException(
                    new gluonj.weaver.CannotCompileException(
                            "Constructor might have an error", e));
        }
    }

}
