/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver;

import gluonj.ast.java.ASTExpr;
import gluonj.ast.java.ASTField;

import java.util.LinkedList;
import java.util.List;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtField;
import javassist.NotFoundException;

public class FieldWeavingVisitor extends IntertypeDeclWeavingVisitor {

    FieldWeavingVisitor(ClassPool pool) {
        super(pool);
    }

    public void atASTField(ASTField field) {

        try {
            int modifier = field.getModifier().getModifiers();
            CtClass type = pool.get(field.getType().getType());
            String name = field.getName();

            if (hasField(cc, name)) {
                addDupInfo(cc.getField(name), field.getInitializer());
            } else {
                CtField cf = new CtField(type, name, cc);
                cf.setModifiers(modifier);

                if (field.getInitializer() == null
                        || field.getInitializer().getExpr() == null
                        || field.getInitializer().getExpr().equals("")) {
                    cc.addField(cf);
                } else {
                    cc.addField(cf, CtField.Initializer.byExpr(field
                            .getInitializer().getExpr()));
                }
            }

        } catch (NotFoundException e) {
            throw new RuntimeException(new CannotCompileException(e));
        } catch (CannotCompileException e) {
            throw new RuntimeException(new CannotCompileException(e));
        }
    }

    private void addDupInfo(CtField cf, ASTExpr initializer) {
        dupInfo.add(new DuplicationInfo(cf, initializer));
    }

    private boolean hasField(CtClass cc, String name) {

        CtField[] cfs = cc.getDeclaredFields();
        for (int i = 0; i < cfs.length; i++) {
            if (cfs[i].getName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    private List dupInfo = new LinkedList();

    public DuplicationInfo[] getDuplication() {
        return (DuplicationInfo[]) dupInfo.toArray(new DuplicationInfo[dupInfo
                .size()]);
    }

    public static class DuplicationInfo {
        private CtField dupField;

        private ASTExpr initializer;

        private DuplicationInfo(CtField cf, ASTExpr i) {
            dupField = cf;
            initializer = i;
        }

        public CtField getDupField() {
            return dupField;
        }

        public ASTExpr getInitializer() {
            return initializer;
        }
    }
}