/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver;

import gluonj.ast.java.ASTConstructor;
import gluonj.ast.java.ASTMethod;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.CtNewConstructor;
import javassist.CtNewMethod;
import javassist.NotFoundException;


class SkeletonWeavingVisitor extends IntertypeDeclWeavingVisitor {


	SkeletonWeavingVisitor(ClassPool pool) {
		super(pool);
	}

	public void atASTMethod(ASTMethod method) {

		try {
			int modifier = method.getModifier().getModifiers();
			CtClass returnType;
			returnType = pool.get(method.getReturnType().getType());

			String name = method.getName();
			CtClass[] parameter = new CtClass[method.getParameter().length];
			CtClass[] throwable = new CtClass[method.getThrowable().length];

			for (int i = 0; i < parameter.length; i++) {
				parameter[i] =
					pool.get(method.getParameter()[i].getType().getType());
			}
			for (int i = 0; i < throwable.length; i++) {
				throwable[i] = pool.get(method.getThrowable()[i].getType());
			}

			CtMethod cm =
				CtNewMethod.abstractMethod(
					returnType,
					name,
					parameter,
					throwable,
					cc);
					
			cm.setModifiers(modifier);
			cc.addMethod(cm);

		} catch (NotFoundException e) {
			throw new RuntimeException(new CannotCompileException(e));
		} catch (CannotCompileException e) {
			throw new RuntimeException(new CannotCompileException(e));
		}
	}

	public void atASTConstructor(ASTConstructor constructor) {

		try {
			int modifier = constructor.getModifier().getModifiers();
			CtClass[] parameter =
				new CtClass[constructor.getParameter().length];
			CtClass[] throwable =
				new CtClass[constructor.getThrowable().length];
			for (int i = 0; i < parameter.length; i++) {
				parameter[i] =
					pool.get(
						constructor.getParameter()[i].getType().getType());
			}
			for (int i = 0; i < throwable.length; i++) {
				throwable[i] =
					pool.get(constructor.getThrowable()[i].getType());
			}
			CtConstructor ccn =
				CtNewConstructor.skeleton(parameter, throwable, cc);
			
			ccn.setModifiers(modifier);
			cc.addConstructor(ccn);

		} catch (NotFoundException e) {
			throw new RuntimeException(new CannotCompileException(e));
		} catch (CannotCompileException e) {
			throw new RuntimeException(new CannotCompileException(e));
		}
	}
}