/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.Advice;
import gluonj.ast.Aspect;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.weaver.CannotCompileException;

public abstract class AbstractProcessor extends Processor {

	public Aspect[] process(Aspect[] aspects) throws CannotCompileException {

		Aspect[] new_aspects = new Aspect[aspects.length];
		for (int i = 0; i < aspects.length; i++) {
			new_aspects[i] = process(aspects[i]);
		}
		return new_aspects;
	}

	protected Aspect process(Aspect aspect) throws CannotCompileException {

		Advice[] advices = aspect.getAdvice();
		Advice[] new_advices = new Advice[advices.length];

		for (int i = 0; i < advices.length; i++) {
			new_advices[i] = process(advices[i]);
		}

		Aspect new_aspect = aspect.replaceAdviceWith(new_advices);
		return new_aspect;
	}

	protected Advice process(Advice advice) throws CannotCompileException {

		ASTPointcutExpr new_expr = process(advice.getPointcutExpr());

		Advice new_advice =
			new Advice(
				advice.getAllParameter(),
				advice.getType(),
				new_expr,
				advice.getBody());

		return new_advice;
	}

	protected ASTPointcutExpr process(ASTPointcutExpr expr) throws CannotCompileException {
		return expr;
	}
}
