/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.Advice;
import gluonj.ast.AdviceType;
import gluonj.ast.Aspect;
import gluonj.ast.intertype.IntertypeDecl;
import gluonj.ast.java.ASTBlock;
import gluonj.ast.java.ASTExpr;
import gluonj.ast.java.ASTField;
import gluonj.ast.java.ASTFormalParameter;
import gluonj.ast.java.ASTMember;
import gluonj.ast.java.ASTMethod;
import gluonj.ast.java.ASTType;
import gluonj.ast.java.Modifier;
import gluonj.ast.java.ext.ASTAnonymousField;
import gluonj.ast.java.ext.ASTCflowType;
import gluonj.ast.pcs.PcsType;
import gluonj.parser.InjectionCodeParser;

import java.util.LinkedList;
import java.util.List;

public class AnonymousFieldProcessor extends Processor {

    public Aspect[] process(Aspect[] aspects) {

        Aspect[] new_aspects = new Aspect[aspects.length];
        for (int i = 0; i < aspects.length; i++) {
            new_aspects[i] = process(aspects[i]);
        }
        return new_aspects;
    }

    private List newAdvice;

    private Aspect process(Aspect aspect) {

        IntertypeDecl[] idecls = aspect.getIntertypeDecl();
        List new_idecls_list = new LinkedList();

        for (int i = 0; i < idecls.length; i++) {
            IntertypeDecl[] id = process(idecls[i]);
            for (int j = 0; j < id.length; j++) {
                new_idecls_list.add(id[j]);
            }
        }
        IntertypeDecl[] new_idecls = new IntertypeDecl[new_idecls_list.size()];
        new_idecls_list.toArray(new_idecls);
        Aspect new_aspect = aspect.replaceIntertypeDeclWith(new_idecls);
        if (newAdvice != null) {

            Advice[] newadv = new Advice[newAdvice.size()
                    + new_aspect.getAdvice().length];

            Advice[] a1 = (Advice[]) newAdvice.toArray(new Advice[newAdvice
                    .size()]);

            System.arraycopy(new_aspect.getAdvice(), 0, newadv, 0, new_aspect
                    .getAdvice().length);
            System.arraycopy(a1, 0, newadv, new_aspect.getAdvice().length,
                    a1.length);
            new_aspect = new_aspect.replaceAdviceWith(newadv);
        }

        return new_aspect;
    } /*
       * <modifier> <type> <target>.aspect = <initializer>; ->
       * 
       * <modifier> <type> <target>.ANONYMOUS$hash;
       * 
       * static <type> <type>.aspectOf( <target> t) { return t.ANONYMOUS$hash; }
       */

    private IntertypeDecl[] process(IntertypeDecl decl) {

        ASTMember m = decl.getMember();
        if (m instanceof ASTAnonymousField) {
            ASTAnonymousField af = (ASTAnonymousField) m;
            PcsType target = decl.getClassPattern();

            ASTField resolved = resolveName(target, af);

            if (af.getParameter().length == 1
                    && target.getSoleClassName().equals("<cflow>")) {
                ASTCflowType type = (ASTCflowType) af.getParameter()[0];
                return makePercflow(target, resolved, type);
            }

            IntertypeDecl[] nameResolvedFields;
            if (af.getParameter().length > 0) {
                nameResolvedFields = changeTypeAndTarget(target, resolved);
            } else {
                nameResolvedFields = new IntertypeDecl[] { new IntertypeDecl(
                        target, resolved) };
            }
            IntertypeDecl[] aspectOf = makeAspectOf(target, af);
            return append(nameResolvedFields, aspectOf);

        } else {
            return new IntertypeDecl[] { decl };
        }
    }

    private IntertypeDecl[] makePercflow(PcsType target, ASTField resolved,
            ASTCflowType type) {

        Modifier m = new Modifier();
        m.setModifier(resolved.getModifier().getModifiers()
                | java.lang.reflect.Modifier.STATIC);

        ASTField newField = new ASTField(m, new ASTType("java.util.List"),
                resolved.getName(), new ASTExpr("new java.util.LinkedList()"));
        PcsType newTarget = InjectionCodeParser.makePcsType(resolved.getType());

        String aspectOfBody = "if(" + resolved.getName()
                + ".size() == 0)  return null;" + "else " + "return ("
                + resolved.getType().getType() + ")" + resolved.getName()
                + ".get(" + resolved.getName() + ".size()-1);";
        String pushBody = resolved.getName() + ".add("
                + resolved.getInitializer().getExpr() + ");";
        String popBody = resolved.getName() + ".remove("
                + resolved.getInitializer().getExpr() + ");";

        Modifier mod = new Modifier();
        mod.setModifier(java.lang.reflect.Modifier.STATIC);
        mod.setModifier(java.lang.reflect.Modifier.PUBLIC);

        ASTMethod aspectOf = new ASTMethod(mod, resolved.getType(), "aspectOf",
                new ASTFormalParameter[] { new ASTFormalParameter(new ASTType(
                        "gluonj.runtime.Cflow"), "cflow") }, new ASTType[0],
                new ASTBlock(aspectOfBody));
        ASTMethod push = new ASTMethod(mod, new ASTType("void"),
                "pushInstance", new ASTFormalParameter[0], new ASTType[0],
                new ASTBlock(pushBody));
        ASTMethod pop = new ASTMethod(mod, new ASTType("void"), "popInstance",
                new ASTFormalParameter[0], new ASTType[0],
                new ASTBlock(popBody));

        newAdvice = new LinkedList();
        newAdvice.add(new Advice(new ASTFormalParameter[0], AdviceType.BEFORE,
                type.getExpr(), new ASTBlock(resolved.getType().getType()
                        + ".pushInstance();")));
        newAdvice.add(new Advice(new ASTFormalParameter[0], AdviceType.AFTER,
                type.getExpr(), new ASTBlock(resolved.getType().getType()
                        + ".popInstance();")));

        return new IntertypeDecl[] {
                new IntertypeDecl(newTarget, newField),
                new IntertypeDecl(newTarget, aspectOf),
                new IntertypeDecl(newTarget, push),
                new IntertypeDecl(newTarget, pop), };
    }

    private IntertypeDecl[] append(IntertypeDecl[] itds, IntertypeDecl[] itds2) {
        IntertypeDecl[] appended = new IntertypeDecl[itds.length + itds2.length];
        System.arraycopy(itds, 0, appended, 0, itds.length);
        System.arraycopy(itds2, 0, appended, itds.length, itds2.length);
        return appended;
    }

    private ASTField resolveName(PcsType targetPat, ASTAnonymousField af) {
        String name = getName(af, targetPat);
        ASTField new_f = new ASTField(af.getModifier(), af.getType(), name, af
                .getInitializer());
        return new_f;
    }

    private IntertypeDecl[] changeTypeAndTarget(PcsType targetPat, ASTField af) {

        ASTField newField = new ASTField(af.getModifier(), new ASTType(
                "java.util.Map"), af.getName(), new ASTExpr(
                "new java.util.HashMap()"));

        return new IntertypeDecl[] { new IntertypeDecl(targetPat, newField) };

    }

    private IntertypeDecl[] makeAspectOf(PcsType targetPat, ASTAnonymousField af) {

        PcsType aspectPat = InjectionCodeParser.makePcsType(af.getType());
        List itdList = new LinkedList();

        Modifier mod = new Modifier();
        mod.setModifier(java.lang.reflect.Modifier.STATIC);

        ASTFormalParameter[] param = new ASTFormalParameter[af.getParameter().length + 1];
        param[0] = new ASTFormalParameter(new ASTType(targetPat
                .getSoleClassName()), "arg0");
        for (int i = 0; i < af.getParameter().length; i++) {
            param[i + 1] = new ASTFormalParameter(af.getParameter()[i], "arg"
                    + (i + 1));
        }
        String body;

        if (af.getParameter().length > 0) {
            body = makeInitializeBody(af, targetPat);
            ASTMethod initialize = new ASTMethod(mod, af.getType(),
                    "initialize", param, new ASTType[0], new ASTBlock(body));
            itdList.add(new IntertypeDecl(aspectPat, initialize));
        }

        body = makeAspectOfBody(af, targetPat);
        ASTMethod aspectOf = new ASTMethod(mod, af.getType(), "aspectOf",
                param, new ASTType[0], new ASTBlock(body));
        itdList.add(new IntertypeDecl(aspectPat, aspectOf));

        return (IntertypeDecl[]) itdList.toArray(new IntertypeDecl[itdList
                .size()]);
    }

    private String makeInitializeBody(ASTAnonymousField af, PcsType targetPat) {

        StringBuffer buf = new StringBuffer();
        buf.append(argsDeclCode(af, targetPat));
        buf.append("return " + af.getInitializer().getExpr() + ";");
        return buf.toString();
    }

    private String argsDeclCode(ASTAnonymousField af, PcsType targetPat) {
        StringBuffer buf = new StringBuffer();
        buf.append(targetPat.getSoleClassName() + " $this = $1;\n");
        for (int i = 0; i < af.getParameter().length; i++) {
            ASTType t = af.getParameter()[i];
            buf.append(t.getType() + " $arg" + i + " = $" + (i + 2) + ";\n");
        }
        return buf.toString();
    }

    private String makeAspectOfBody(ASTAnonymousField af, PcsType pat) {
        if (af.getParameter().length == 0) {
            return "return $1." + getName(af, pat) + ";";
        }
        int size = af.getParameter().length + 1;
        StringBuffer buf = new StringBuffer();
        buf.append("java.util.Map m;");
        buf.append("m = $1." + getName(af, pat) + ";");

        for (int i = 2; i < size; i++) {
            buf.append(mapCode(i, "m", af.getType().getType(), false));
        }
        buf.append(mapCode(size, "m", af.getType().getType(), true));
        return buf.toString();
    }

    private String mapCode(int index, String mapName, String returnType,
            boolean last) {

        String arg = "$" + index;
        String init;
        String getCode;
        if (last) {
            StringBuffer args = new StringBuffer("$1");
            for (int i = 2; i <= index; i++) {
                args.append(", $" + i);
            }
            init = "initialize(" + args + ")";
            getCode = "return (" + returnType + ")m.get(" + arg + ");";
        } else {
            init = "new java.util.HashMap()";
            getCode = "m = (java.util.Map)m.get($" + arg + ");";
        }
        StringBuffer buf = new StringBuffer();
        buf.append("if(!m.containsKey(" + arg + ")) {");
        buf.append("    m.put(" + arg + ", " + init + ");");
        buf.append("}");
        buf.append(getCode);
        return buf.toString();
    }

    private String getName(ASTAnonymousField f, PcsType pat) {
        String postFix = "";
        if(pat.getSoleClassName().equals("<cflow>")) {
            postFix = gluonj.runtime.Aspect.PERCFLOW;
        }
        if(f.getParameter().length > 0) {
            postFix = gluonj.runtime.Aspect.PEROBJECTS;
        }
        
        return gluonj.runtime.Aspect.getAFPreFix(f.getType().toString(), pat
                .toString()) + postFix;
    }
}