/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.Advice;
import gluonj.ast.AdviceType;
import gluonj.ast.Aspect;
import gluonj.ast.PointcutDecl;
import gluonj.ast.java.ASTBlock;
import gluonj.ast.java.ASTFormalParameter;
import gluonj.ast.pcs.IfResidue;
import gluonj.ast.pcs.PointcutShadow;
import gluonj.ast.pointcut.ASTCflow;
import gluonj.ast.pointcut.ASTCflowBelow;
import gluonj.ast.pointcut.ASTPCShadow;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.visitor.PcPrimitiveTranslator;
import gluonj.ast.pointcut.visitor.Traverser;
import gluonj.weaver.JoinPointConstants;

import java.util.LinkedList;
import java.util.List;

public class CflowRemover extends Processor {

    private List flagChangingAdviceList;

    public CflowRemover() {
    }

    public Aspect[] process(Aspect[] aspects) {

        Aspect[] new_aspects = new Aspect[aspects.length];
        for (int i = 0; i < aspects.length; i++) {
            new_aspects[i] = process(aspects[i]);
        }
        return new_aspects;
    }

    private Aspect process(Aspect aspect) {

        PointcutDecl[] pcdecls = aspect.getPointcutDecl();
        Advice[] advices = aspect.getAdvice();

        List newAdvs = new LinkedList();
        List newPCDecls = new LinkedList();

        CflowVisitor v = new CflowVisitor();

        for (int i = 0; i < advices.length; i++) {
            newAdvs.add(process(advices[i], v));
        }
        for (int i = 0; i < pcdecls.length; i++) {
            newPCDecls.add(process(pcdecls[i], v));
        }

        newAdvs.addAll(0, v.getCflowBelowAdvs());
        newAdvs.addAll(v.getCflowAdvs());

        Aspect new_aspect = aspect.replaceAdviceWith((Advice[]) newAdvs
                .toArray(new Advice[newAdvs.size()]));
        new_aspect = new_aspect
                .replacePointcutDeclWith((PointcutDecl[]) newPCDecls
                        .toArray(new PointcutDecl[newPCDecls.size()]));
        return new_aspect;
    }

    /*
     * modify: flagChagingAdviceList
     */

    private PointcutDecl process(PointcutDecl decl, CflowVisitor v) {
        Traverser.traverseAndOrNotPrimitive(decl.getPointcut(), v);
        ASTPointcutExpr newexpr = v.getNewPC();
        return new PointcutDecl(decl.getName(), decl.getParametersList(),
                newexpr);
    }

    /*
     * modify: flagChagingAdviceList
     */
    private Advice process(Advice advice, CflowVisitor v) {
        Traverser.traverseAndOrNotPrimitive(advice.getPointcutExpr(), v);
        ASTPointcutExpr newexpr = v.getNewPC();
        return new Advice(advice.getAllParameter(), advice.getType(), newexpr,
                advice.getBody());
    }

    static class CflowVisitor extends PcPrimitiveTranslator {

        private List cflowBelowAdvs = new LinkedList();

        private List cflowAdvs = new LinkedList();

        
        public ASTPointcutExpr changePcPrimitive(ASTPointcutPrimitive a) {
            if (a.getType() == JoinPointConstants.CFLOW) {
                return changeASTCflow((ASTCflow) a);
            } else if (a.getType() == JoinPointConstants.CFLOW_BELOW) {
                return changeASTCflowBelow((ASTCflowBelow) a);
            } else {
                return a;
            }
        }

        private ASTPointcutExpr changeASTCflowBelow(ASTCflowBelow expr) {
            int flagid = expr.hashCode();
            return changeCflow(flagid, expr.getExpr(), cflowBelowAdvs);
        }

        public ASTPointcutExpr changeASTCflow(ASTCflow expr) {
            int flagid = expr.hashCode();
            return changeCflow(flagid, expr.getExpr(), cflowAdvs);
        }

        private ASTPointcutExpr changeCflow(int flagid, ASTPointcutExpr expr,
                List advs) {
            String condition = "gluonj.runtime.CflowCounter.isIn(" + flagid
                    + ")";
            IfResidue r = new IfResidue(condition);

            Advice in = new Advice(new ASTFormalParameter[0],
                    AdviceType.BEFORE, expr, new ASTBlock(
                            "gluonj.runtime.CflowCounter.enter(" + flagid
                                    + ");"));
            Advice out = new Advice(new ASTFormalParameter[0],
                    AdviceType.AFTER, expr, new ASTBlock(
                            "gluonj.runtime.CflowCounter.getout(" + flagid
                                    + ");"));
            advs.add(in);
            advs.add(out);

            return new ASTPCShadow(PointcutShadow.TRUE, r);

        }

        public List getCflowBelowAdvs() {
            return cflowBelowAdvs;
        }

        public List getCflowAdvs() {
            return cflowAdvs;
        }
    }
}