/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.PointcutExprUtil;
import gluonj.ast.pointcut.visitor.Traverser;
import gluonj.ast.pointcut.visitor.Visitor;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

public class DisjunctiveNFProcessor extends AbstractProcessor {

    protected ASTPointcutExpr process(ASTPointcutExpr expr) {
        DNFVisitor v = new DNFVisitor();
        Traverser.traverseAndOrNotPrimitive(expr, v);
        return v.getNewExpr();
    }

    private class DNFVisitor extends Visitor {

        /*
         * ϊDNF ςłX^bN ^ Stack <List <List <PointcutExpr>>
         * 
         * X^bN̗vfDNF \2̃XgBႦ΁A { {A1, A2, A3}, {B1}, {C1, C2}}
         * Ƃ2̃XǵA (A1 && A2 && A3) || B1 || (C1 && C2) Ƃ DNF \
         */
        private Stack pcListStack = new Stack();

        /*
         * not tOB̂Ƃ]Ă|CgJbgwq not Ȃ true not
         * |CgJbg]邽тɒl]
         */
        private boolean flag = false;

        /*
         * ϊDNF Ԃ
         */
        public ASTPointcutExpr getNewExpr() {
            return PointcutExprUtil.makeDisjunctive((List) pcListStack.pop());
        }

        /*
         * not tO𔽓]
         */
        public void beforeASTNot(ASTNot expr) {
            flag = !flag;
        }

        /*
         * ]not tO̒lɖ߂
         */
        public void afterASTNot(ASTNot expr) {
            flag = !flag;
        }

        /*
         * |CgJbgwqvisit ƂɌĂ΂ not tOĂAStPcPrimitive not tOtrue
         * ɂ pcStack  { { pcPrimitive } } push
         */
        public void atPointcutPrimitive(ASTPointcutPrimitive expr) {


        }

        /*
         * ASTAnd ̎q|CgJbgvisit ɌĂ΂ not tOĂꍇ or Ƃĕ]
         * 
         * !(A and B) -> !A !and !B -> !A or !B
         */
        public void afterASTAnd(ASTAnd expr) {
            if (flag) {
                or();
            } else {
                and();
            }
        }

        /*
         * ASTOr ̎q|CgJbgvisit ɌĂ΂ not tOĂꍇ and Ƃĕ]
         * 
         * !(A or B) -> !A !or !B -> !A and !B
         */
        public void afterASTOr(ASTOr expr) {
            if (flag) {
                and();
            } else {
                or();
            }
        }

        /*
         * 2DNF  and ŌԁB
         * 
         * ̓Iɂand Ō΂ꂽȉDNF A
         * 
         * (A1 || A2 || ... || An) && (B1 || B2 || ... || Bm)
         * 
         * ̂悤ɕϊB
         * 
         * (A1 && B1) || (A1 && B2) || ... || (A1 && Bm) || (A2 && B1) || (A2 &&
         * B2) || ... || (A2 && Bm) || ... || (An && B1) || (An && B2) || ... ||
         * (An && Bm)
         * 
         * A1 , ... , An , B1, ... , Bm  and ݂̂łȂꂽ_
         */
        private void and() {

            /*
             * list2 <- (A1 || A2 || ... || An)
             */
            List list2 = (List) pcListStack.pop();
            /*
             * list1 <- (B1 || B2 || ... || Bm)
             */
            List list1 = (List) pcListStack.pop();

            List newList = new ArrayList();

            for (Iterator iter1 = list1.iterator(); iter1.hasNext();) {
                List primitives1 = (List) iter1.next();
                for (Iterator iter2 = list2.iterator(); iter2.hasNext();) {
                    List primitives2 = (List) iter2.next();

                    /*
                     * newPrimitives <- (Ai && Bj)
                     */
                    List newPrimitives = new LinkedList();
                    newPrimitives.addAll(primitives1);
                    newPrimitives.addAll(primitives2);
                    newList.add(newPrimitives);
                }
            }
            pcListStack.push(newList);
        }

        /*
         * 2DNF  or ŌԁB
         * 
         * ̓Iɂor Ō΂ꂽȉDNF A
         * 
         * (A1 || A2 || ... || An) || (B1 || B2 || ... || Bm)
         * 
         * ̂悤ɕϊB
         * 
         * (A1 || A2 || ... || An || B1 || B2 || ... || Bm)
         * 
         * A1 , ... , An , B1, ... , Bm  and ݂̂łȂꂽ_
         */
        private void or() {

            /*
             * list2 <- (A1 || A2 || ... || An)
             */
            List list2 = (List) pcListStack.pop();
            /*
             * list1 <- (B1 || B2 || ... || Bm)
             */
            List list1 = (List) pcListStack.pop();

            list1.addAll(list2);
            pcListStack.push(list1);
        }
        
        private void push(ASTPointcutPrimitive expr) {
			List l = new ArrayList();
			List m = new LinkedList();
			l.add(m);
			if(flag) {
				m.add(new ASTNot(expr));
			} else {
				m.add(expr);
			}
			pcListStack.push(l);
        }
        
        public void atAstPointcutPrimitive(ASTPointcutPrimitive expr) {
            push(expr);
        }
    }
}
