/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTArgs;
import gluonj.ast.pointcut.ASTCflow;
import gluonj.ast.pointcut.ASTFieldGet;
import gluonj.ast.pointcut.ASTFieldSet;
import gluonj.ast.pointcut.ASTMethodCall;
import gluonj.ast.pointcut.ASTMethodExecution;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPCReference;
import gluonj.ast.pointcut.ASTPCShadow;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.ASTResidue;
import gluonj.ast.pointcut.ASTShadow;
import gluonj.ast.pointcut.ASTTarget;
import gluonj.ast.pointcut.ASTThis;
import gluonj.ast.pointcut.ASTWithin;
import gluonj.ast.pointcut.ASTWithincode;
import gluonj.ast.pointcut.visitor.Visitor;

import java.util.LinkedList;
import java.util.List;


public abstract class PointcutTranslationVisitor extends Visitor {

    List stack = new LinkedList();
    
    protected ASTPointcutExpr pop() {
        ASTPointcutExpr e = (ASTPointcutExpr) stack.get(0);
        stack.remove(0);
        return e;
    }

    protected void push(ASTPointcutExpr expr) {
        stack.add(0, expr);
    }

    public ASTPointcutExpr getNewExpr() {
        return pop();
    }

    public void afterASTAnd(ASTAnd expr) {
        ASTPointcutExpr new_right = pop();
        ASTPointcutExpr new_left = pop();
        push(new ASTAnd(new_left, new_right));
    }

    public void afterASTNot(ASTNot expr) {
        push(new ASTNot(pop()));
    }

    public void afterASTOr(ASTOr expr) {
        ASTPointcutExpr new_right = pop();
        ASTPointcutExpr new_left = pop();
        push(new ASTOr(new_left, new_right));
    }

    public void atASTArgs(ASTArgs expr) {
        push(expr);
    }

    public void atASTCflow(ASTCflow expr) {
        push(expr);
    }

    public void atASTFieldGet(ASTFieldGet expr) {
        push(expr);
    }

    public void atASTFieldSet(ASTFieldSet expr) {
        push(expr);
    }

    public void atASTMethodCall(ASTMethodCall expr) {
        push(expr);
    }

    public void atASTMethodExecution(ASTMethodExecution expr) {
        push(expr);
    }
    public void atPointcutPrimitive(ASTPointcutPrimitive expr) {
        push(expr);
    }
    public void atASTPCReference(ASTPCReference expr) {
        push(expr);
    }

    public void atASTResidue(ASTResidue expr) {
        push(expr);
    }

    public void atASTTarget(ASTTarget expr) {
        push(expr);
    }

    public void atASTThis(ASTThis expr) {
        push(expr);
    }

    public void atASTWithin(ASTWithin expr) {
        push(expr);
    }

    public void atASTWithincode(ASTWithincode expr) {
        push(expr);
    }
    public void atASTShadow(ASTShadow shadow) {
        push(shadow);
    }
    public void atASTCompieldPC(ASTPCShadow compiledPC) {
        push(compiledPC);
    }
    
    public final void beforeASTAnd(ASTAnd expr) {
        
    }
    public final void beforeASTNot(ASTNot expr) {
        
    }
    public final void beforeASTOr(ASTOr expr) {
        
    }
}
