/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.pcs.PcsCall;
import gluonj.ast.pcs.PcsNot;
import gluonj.ast.pcs.PcsSet;
import gluonj.ast.pcs.PointcutResidue;
import gluonj.ast.pcs.PointcutShadow;
import gluonj.ast.pointcut.ASTFieldGet;
import gluonj.ast.pointcut.ASTFieldSet;
import gluonj.ast.pointcut.ASTMethodCall;
import gluonj.ast.pointcut.ASTMethodExecution;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTPCShadow;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.ASTUnaryExpr;
import gluonj.ast.pointcut.visitor.DNFTranslator;
import gluonj.ast.pointcut.visitor.Traverser;
import gluonj.weaver.JoinPointConstants;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/*
 * input: DNF consists of Kinded Pointcut , ASTShadow or ASTResidue
 * 
 * output : DNF. the conjunctive expression has only one ASTShadow object, or
 * two objects: one is an ASTShadow object and the other is an ASTResidue
 * object.
 *  
 */
public class ShadowProcessor extends AbstractProcessor {

    protected ASTPointcutExpr process(ASTPointcutExpr expr) {

            ShadowRemover s = new ShadowRemover();
            Traverser.traverseDNF(expr, s);
            return s.getDNF();
    }

    private static class ShadowRemover extends DNFTranslator {
        public List[] translateCNF(List exprs) {
            List newCNF = new LinkedList();

            for (Iterator iter = exprs.iterator(); iter.hasNext();) {

                ASTUnaryExpr e = (ASTUnaryExpr) iter.next();
                int type = e.getType();
                ASTPointcutExpr newExpr;

                if (type == JoinPointConstants.NOT) {
                    newExpr = changeNot((ASTNot) e);
                } else {
                    newExpr = changePcPrimitive((ASTPointcutPrimitive) e, false);
                }
                newCNF.add(newExpr);
            }
            return new List[] { newCNF };
        }

        public ASTPointcutExpr changeNot(ASTNot not) {
            ASTPointcutPrimitive e = (ASTPointcutPrimitive) not.getChild();
            return changePcPrimitive(e, true);
        }

        public ASTPointcutExpr changePcPrimitive(ASTPointcutPrimitive p,
                boolean isNot) {

            int type = p.getType();
            if (type == JoinPointConstants.EXECUTION) {
                return changeExec((ASTMethodExecution) p, isNot);
            } else if (type == JoinPointConstants.CALL) {
                return changeCall((ASTMethodCall) p, isNot);
            } else if (type == JoinPointConstants.FIELD_GET) {
                return changeGet((ASTFieldGet) p, isNot);
            } else if (type == JoinPointConstants.FIELD_SET) {
                return changeSet((ASTFieldSet) p, isNot);
            } else {
                if (isNot) {
                    return new ASTNot(p);
                } else {
                    return p;
                }
            }
        }

        private ASTPointcutExpr changeSet(ASTFieldSet expr, boolean isNot) {
            PointcutShadow p = setNot(new PcsSet(expr.getPattern()), isNot);
            return new ASTPCShadow(p, PointcutResidue.TRUE);
        }

        private ASTPointcutExpr changeGet(ASTFieldGet expr, boolean isNot) {
            PointcutShadow p = setNot(new PcsSet(expr.getPattern()), isNot);
            return new ASTPCShadow(p, PointcutResidue.TRUE);
        }

        private ASTPointcutExpr changeCall(ASTMethodCall expr, boolean isNot) {
            PointcutShadow p = setNot(new PcsCall(expr.getPattern()), isNot);
            return new ASTPCShadow(p, PointcutResidue.TRUE);
        }

        private ASTPointcutExpr changeExec(ASTMethodExecution expr,
                boolean isNot) {
            PointcutShadow p = setNot(expr.getPattern(), isNot);
            return new ASTPCShadow(p, PointcutResidue.TRUE);
        }

        private PointcutShadow setNot(PointcutShadow p, boolean isNot) {
            if (isNot) {
                return new PcsNot(p);
            } else {
                return p;
            }
        }
    }
}
