/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.pcs.PcsBehavior;
import gluonj.ast.pcs.PcsExpr;
import gluonj.ast.pcs.PcsModifier;
import gluonj.ast.pcs.PcsNot;
import gluonj.ast.pcs.PcsOr;
import gluonj.ast.pcs.PcsParameter;
import gluonj.ast.pcs.PcsParameterDotDot;
import gluonj.ast.pcs.PcsType;
import gluonj.ast.pcs.PointcutResidue;
import gluonj.ast.pcs.PointcutShadow;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTPCShadow;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;
import gluonj.ast.pointcut.ASTUnaryExpr;
import gluonj.ast.pointcut.ASTWithin;
import gluonj.ast.pointcut.ASTWithincode;
import gluonj.ast.pointcut.visitor.DNFTranslator;
import gluonj.ast.pointcut.visitor.Traverser;
import gluonj.weaver.JoinPointConstants;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class WithinRemover extends AbstractProcessor {

    protected ASTPointcutExpr process(ASTPointcutExpr expr) {

        WithinTranslator t = new WithinTranslator();
        Traverser.traverseDNF(expr, t);
        return t.getDNF();
    }

    private class WithinTranslator extends DNFTranslator {
        public List[] translateCNF(List exprs) {
            List newCNF = new LinkedList();

            for (Iterator iter = exprs.iterator(); iter.hasNext();) {

                ASTUnaryExpr e = (ASTUnaryExpr) iter.next();
                int type = e.getType();
                ASTPointcutExpr newExpr;

                if (type == JoinPointConstants.NOT) {
                    newExpr = changeNot((ASTNot) e);
                } else {
                    newExpr = changePcPrimitive((ASTPointcutPrimitive) e, false);
                }
                newCNF.add(newExpr);
            }
            return new List[] { newCNF };
        }

        public void atAstUnaryExpr(ASTUnaryExpr expr) {
            if (expr.getType() == JoinPointConstants.NOT) {
                changeNot((ASTNot) expr);
            } else {
                changePcPrimitive((ASTPointcutPrimitive) expr, false);
            }
        }

        public ASTPointcutExpr changeNot(ASTNot not) {
            ASTPointcutPrimitive e = (ASTPointcutPrimitive) not.getChild();
            return changePcPrimitive(e, true);
        }

        private ASTPointcutExpr changePcPrimitive(ASTPointcutPrimitive e,
                boolean isNot) {
            if (e.getType() == JoinPointConstants.WITHIN) {
                return changeWithin((ASTWithin) e, isNot);
            } else if (e.getType() == JoinPointConstants.WITHINCODE) {
                return changeWithinCode((ASTWithincode) e, isNot);
            } else {
                if (isNot) {
                    return new ASTNot(e);
                } else {
                    return e;
                }
            }
        }

        private ASTPointcutExpr changeWithinCode(ASTWithincode expr,
                boolean isNot) {
            PointcutShadow s = makeShadow(expr.getPattern(), isNot);
            return new ASTPCShadow(s, PointcutResidue.TRUE);
        }

        private ASTPointcutExpr changeWithin(ASTWithin expr, boolean isNot) {

            PcsBehavior where = new PcsBehavior(new PcsModifier(), expr
                    .getPattern(),
                    new PcsParameter[] { new PcsParameterDotDot() },
                    new PcsType[0]);

            PointcutShadow s = makeShadow(where, isNot);
            return new ASTPCShadow(s, PointcutResidue.TRUE);
        }

        private PointcutShadow makeShadow(PcsBehavior where, boolean isNot) {
            PointcutShadow s = new PcsOr(new PcsExpr(where), where);
            if (isNot) {
                s = new PcsNot(s);
            }
            return s;
        }

    }
}
