/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ast.pointcut.visitor;

import gluonj.ast.pcs.PcsBehavior;
import gluonj.ast.pcs.PcsField;
import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTBinaryExpr;
import gluonj.ast.pointcut.ASTFieldGet;
import gluonj.ast.pointcut.ASTFieldSet;
import gluonj.ast.pointcut.ASTMethodCall;
import gluonj.ast.pointcut.ASTMethodExecution;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTPointcutPrimitive;

import java.util.List;

import junit.framework.TestCase;

public class TestTranslator extends TestCase {

    // call(* *.*(..))
    ASTPointcutExpr expr1 = new ASTMethodCall(PcsBehavior.ANY);

    // call(* *.*(..)) && execution(* *.*(..))
    ASTPointcutExpr expr2 = new ASTAnd(new ASTMethodCall(PcsBehavior.ANY),
            new ASTMethodExecution(PcsBehavior.ANY));

    // call(* *.*(..)) || execution(* *.*(..))
    ASTPointcutExpr expr3 = new ASTOr(new ASTMethodCall(PcsBehavior.ANY),
            new ASTMethodExecution(PcsBehavior.ANY));

    // (call(* *.*(..)) && execution(* *.*(..))) ||
    // (get(* *.*) && set(* *.*))
    ASTPointcutExpr expr4 = new ASTOr(new ASTAnd(new ASTMethodCall(
            PcsBehavior.ANY), new ASTMethodExecution(PcsBehavior.ANY)),
            new ASTAnd(new ASTFieldGet(PcsField.ANY), new ASTFieldSet(
                    PcsField.ANY)));

    class DNFNop extends DNFTranslator {
        public List[] translateCNF(List exprs) {
            return new List[] { exprs };
        }
    }

    public final void testDNFTranslator1() {
        DNFTranslator nop = new DNFNop();
        Traverser.traverseDNF(expr1, nop);
        assertTrue(nop.getDNF() == expr1);
    }

    public final void testDNFTranslator2() {
        DNFTranslator nop = new DNFNop();
        Traverser.traverseDNF(expr2, nop);
        assertTrue(areSame(nop.getDNF(), expr2));
    }

    public final void testDNFTranslator3() {
        DNFTranslator nop = new DNFNop();
        Traverser.traverseDNF(expr3, nop);
        assertTrue(areSame(nop.getDNF(), expr3));
    }

    public final void testDNFTranslator4() {
        DNFTranslator nop = new DNFNop();
        Traverser.traverseDNF(expr4, nop);
        assertTrue(areSame(nop.getDNF(), expr4));
    }

    class PriNop extends PcPrimitiveTranslator {

        public ASTPointcutExpr changePcPrimitive(ASTPointcutPrimitive a) {
            return a;
        }
    }

    public final void testPrimitiveTranslator() {
        PcPrimitiveTranslator nop = new PriNop();
        Traverser.traverseAndOrNotPrimitive(expr1, nop);
        assertTrue(areSame(nop.getNewPC(), expr1));
    }
    public final void testPrimitiveTranslator2() {
        PcPrimitiveTranslator nop = new PriNop();
        Traverser.traverseAndOrNotPrimitive(expr2, nop);
        assertTrue(areSame(nop.getNewPC(), expr2));
    }
    public final void testPrimitiveTranslator3() {
        PcPrimitiveTranslator nop = new PriNop();
        Traverser.traverseAndOrNotPrimitive(expr3, nop);
        assertTrue(areSame(nop.getNewPC(), expr3));
    }
    public final void testPrimitiveTranslator4() {
        PcPrimitiveTranslator nop = new PriNop();
        Traverser.traverseAndOrNotPrimitive(expr4, nop);
        assertTrue(areSame(nop.getNewPC(), expr4));
    }

    boolean areSame(ASTPointcutExpr e1, ASTPointcutExpr e2) {

        if (e1.getType() != e2.getType())
            return false;
        if (e1 instanceof ASTBinaryExpr) {
            ASTBinaryExpr b1 = (ASTBinaryExpr) e1;
            ASTBinaryExpr b2 = (ASTBinaryExpr) e2;
            if (areSame(b1.getLeft(), b2.getLeft())
                    && areSame(b1.getRight(), b2.getRight())) {
                return true;
            } else {
                return false;
            }
        } else {
            boolean b = e1 == e2;
            return b;
        }
    }
}