/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.test;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import test.NotMatchedException;


public class FlagRunner extends Runner {

    public static void main(String[] args) throws IllegalArgumentException,
            SecurityException, ClassNotFoundException,
            InvocationTargetException, IllegalAccessException,
            InstantiationException, NoSuchFieldException {
        new FlagRunner().doMain(args);
    }

    protected void invoke(Method method, Class cls)
            throws IllegalArgumentException, IllegalAccessException,
            InvocationTargetException, InstantiationException,
            SecurityException, NoSuchFieldException {

        Class[] paramTypes = method.getParameterTypes();
        if (paramTypes.length == 1 && paramTypes[0].getName().equals("[Z")) {

            String fieldName = method.getName() + "_flagnum";
            Field fFlagNum = cls.getDeclaredField(fieldName);
            int flagNum = ((Integer) fFlagNum.get(cls.newInstance()))
                    .intValue();
            invoke(method, cls, flagNum);
        }
    }

    private void invoke(Method method, Class cls, int flagnum)
            throws IllegalArgumentException, IllegalAccessException,
            InvocationTargetException, InstantiationException {

        boolean[][] flagSet = makeFlagSet(flagnum);
        for (int i = 0; i < Math.pow(2, flagnum) - 1; i++) {
            boolean thrown = false;
            try {
                method.invoke(cls.newInstance(), new Object[] { flagSet[i] });
            } catch (InvocationTargetException e) {
                if (e.getCause() instanceof NotMatchedException) {
                    thrown = true;
                }
            }
            if (!thrown) {
                throw new RuntimeException();
            }
        }

        method.invoke(cls.newInstance(), new Object[] { flagSet[(int) Math.pow(
                2, flagnum) - 1] });
    }

    private boolean[][] makeFlagSet(int size) {

        boolean[][] flags = new boolean[(int) Math.pow(2, size)][size];
        for (int i = 0; i < Math.pow(2, size); i++) {
            for (int j = 0; j < size; j++) {
                flags[i][j] = ((i >> j) % 2 != 0);
            }
        }
        return flags;
    }

}