/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.ui;

import gluonj.NotFoundException;

import java.io.File;
import java.util.Arrays;

import junit.framework.TestCase;

public class TestCommand extends TestCase {

    public final void testOptionTable1() {

        OptionTable.Factory factory = new OptionTable.Factory();
        OptionTable table = factory.make(new String[] { "arg1" });

        String[] args = table.getArgument();
        assertTrue(args.length == 1);
        assertTrue(args[0].equals("arg1"));
    }

    public final void testOptionTable2() {
        OptionTable.Factory factory = new OptionTable.Factory();
        factory.setOptionFormat("a", 1);
        OptionTable table = factory.make(new String[] { "-a", "val1", "arg1",
                "arg2" });

        String[] opt_a = table.getValueOf("a");
        String[] args = table.getArgument();

        assertTrue(opt_a.length == 1);
        assertTrue(opt_a[0].equals("val1"));

        assertTrue(args.length == 2);
        assertTrue(args[0].equals("arg1"));
        assertTrue(args[1].equals("arg2"));
    }

    public final void testFileFinderNoWildCard() {
        String target = "usr/test/Foo";
        String pattern = "usr/test/Foo";
        assertTrue(new FileFinder().match(pattern, target));

    }

    public final void testFileFinderWithWildCard1() {
        String pattern = "usr/test/Fo*";
        String target = "usr/test/Fo";
        String target2 = "usr/test/Fod";
        String target3 = "usr/test/Food";
        assertTrue(new FileFinder().match(pattern, target));
        assertTrue(new FileFinder().match(pattern, target2));

        String pattern2 = "usr/test/Fo*d";
        assertFalse(new FileFinder().match(pattern2, target));
        assertTrue(new FileFinder().match(pattern2, target2));
        assertTrue(new FileFinder().match(pattern2, target3));

    }

    public final void testFileFinderWithWildCard2() {

        String pattern = "usr/*/Foo";
        String target = "usr/a/Foo";
        String target2 = "usr/b/Foo";
        String target3 = "usr/Foo";

        assertTrue(new FileFinder().match(pattern, target));
        assertTrue(new FileFinder().match(pattern, target2));
        assertFalse(new FileFinder().match(pattern, target3));
    }

    public final void testFileFinderWithWildCard3() {
        /*
         * String pattern = "usr/**"; String target = "usr/Foo"; String target2 =
         * "usr/b/Bar"; String target3 = "usr";
         * 
         * assertTrue(new FileFinder().match(pattern, target)); assertTrue(new
         * FileFinder().match(pattern, target2)); assertFalse(new
         * FileFinder().match(pattern, target3));
         */
    }

    public final void testFileFinderWithWildCard4() {

        String pattern = "usr/**/Main";
        String target = "usr/Main";
        String target2 = "usr/foo/Main";
        String target3 = "usr/Main/Main";
        String target4 = "usr/Main/Main2";

        assertTrue(new FileFinder().match(pattern, target));
        assertTrue(new FileFinder().match(pattern, target2));
        assertTrue(new FileFinder().match(pattern, target3));
        assertFalse(new FileFinder().match(pattern, target4));
    }

    public final void testJar1() {

        try {
            String[] classes = new FileFinder().resolveClassPatInJar(
                    "gluonj/cui/A", "./test/gluonj/ui/test.jar");

            assertTrue(classes.length == 1);
            assertTrue(classes[0].equals("gluonj.cui.A"));

        } catch (NotFoundException e) {
            assertTrue(false);
        }
    }

    public final void testJar2() {

        try {
            String[] classes = new FileFinder().resolveClassPatInJar(
                    "gluonj/cui/*", "./test/gluonj/ui/test.jar");

            assertEquals(3, classes.length);
            assertEquals("gluonj.cui.A", classes[0]);
            assertEquals("gluonj.cui.B", classes[1]);
            assertEquals("gluonj.cui.A$C", classes[2]);

        } catch (NotFoundException e) {
            assertTrue(false);
        }
    }

    public final void testJar3() {
        try {
            String[] classes = new FileFinder().resolveClassPatInJar(
                    "gluonj/**/A", "./test/gluonj/ui/test.jar");

            assertTrue(classes.length == 2);
            assertTrue(classes[0].equals("gluonj.cui.A"));
            assertTrue(classes[1].equals("gluonj.cui.test.A"));

        } catch (NotFoundException e) {
            assertTrue(false);
        }
    }

    public final void testJarNotFound() {
        try {
            String[] classes = new FileFinder().resolveClassPatInJar(
                    "gluonj/ui/A", "./test/gluonj/ui/not.jar");

            assertTrue(false);
        } catch (NotFoundException e) {
            assertTrue(true);
        }
    }

    public final void testDir1() {
        String current = new File(".").getAbsolutePath();
        String[] classes = new FileFinder().resolveClassPatInDir(
                "gluonj/ui/test/A", current + File.separator + "bin");
        assertEquals(classes.length, 1);
        assertEquals(classes[0],("gluonj.ui.test.A"));
    }

    public final void testDir2() {
        String current = new File(".").getAbsolutePath();
        String[] classes = new FileFinder().resolveClassPatInDir(
                "gluonj/ui/test/*", current + File.separator + "bin");
        Arrays.sort(classes);
        assertEquals(classes.length, 3);
        assertEquals(classes[0],("gluonj.ui.test.A"));
        assertEquals(classes[1],("gluonj.ui.test.A$C"));
        assertEquals(classes[2],("gluonj.ui.test.B"));
    }

    public final void testDir3() {
        String current = new File(".").getAbsolutePath();
        String[] classes = new FileFinder().resolveClassPatInDir(
                "gluonj/ui/**/A", current + File.separator + "bin");
        Arrays.sort(classes);
        assertEquals(classes.length , 2);
        assertEquals(classes[0],("gluonj.ui.test.A"));
        assertEquals(classes[1],("gluonj.ui.test.test.A"));
    }
}