/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver;


import java.lang.reflect.Method;

import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.Loader;
import test.GluonJTestCase;

public class TestClosureMaker extends GluonJTestCase {

    /**
     * A1.getStrNull() ̒g A1.getStrVal() ĂяoClosure Œu
     */
    public void testMakingClosure1() throws Exception {

        ClassPool pool = getPool();
        ClosureMaker maker = new ClosureMaker(pool);
        CtClass ccA = pool.get("gluonj.weaver.A1");
        CtClass cccl = maker.attachClosure(ccA, ccA
                .getDeclaredMethod("getStrVal"));
        CtMethod ccGet = ccA.getDeclaredMethod("getStrNull");
        String body = "return (String) new " + cccl.getName()
                + "().proceed(this, new Object[0]);";
        ccGet.setBody(body);

        Loader l = new Loader(pool);
        l.loadClass(cccl.getName());
        Class ca = l.loadClass(ccA.getName());

        Method m = ca.getMethod("getStrNull", new Class[0]);
        String r = (String) m.invoke(ca.newInstance(), new Object[0]);
        assertTrue(r.equals("A1"));
    }

    /**
     * A1.getIntZero() ̒g A1.getIntNull() Ăяo Closure ĂяoŒu
     */
    public void testMakingClosure2() throws Exception {

        ClassPool pool = getPool();
        ClosureMaker maker = new ClosureMaker(pool);

        CtClass ccA = pool.get("gluonj.weaver.A1");
        CtClass cccl = maker.attachClosure(ccA, ccA
                .getDeclaredMethod("getIntVal"));
        CtMethod ccGet = ccA.getDeclaredMethod("getIntZero");
        String body = "return ((Integer) new " + cccl.getName()
                + "().proceed(this, new Object[0])).intValue();";
        ccGet.setBody(body);

        Loader l = new Loader(pool);
        l.loadClass(cccl.getName());
        Class ca = l.loadClass(ccA.getName());

        Method m = ca.getMethod("getIntZero", new Class[0]);
        int r = ((Integer) m.invoke(ca.newInstance(), new Object[0]))
                .intValue();
        assertTrue(r == 1);
    }

    /**
     * A1.nop() ̒gA1.succ() Œu
     */
    public void testMakingClosure3() throws Exception {

        ClassPool pool = getPool();
        ClosureMaker maker = new ClosureMaker(pool);

        CtClass ccA = pool.get("gluonj.weaver.A1");
        CtClass cccl = maker.attachClosure(ccA, ccA.getDeclaredMethod("succX"));
        CtMethod ccGet = ccA.getDeclaredMethod("nop");
        String body = "new " + cccl.getName()
                + "().proceed(this, new Object[0]);";
        ccGet.setBody(body);

        Loader l = new Loader(pool);
        l.loadClass(cccl.getName());
        Class ca = l.loadClass(ccA.getName());

        Method m = ca.getMethod("nop", new Class[0]);
        Object a = ca.newInstance();
        m.invoke(a, new Object[0]);
        assertTrue(((Integer) ca.getField("x").get(a)).intValue() == 1);
    }

    public void tesLongCast() throws Exception {
        ClassPool pool = getPool();
        
        CtMethod[] c = pool.get("java.lang.Thread").getDeclaredMethods();
        for(int i=0;i<c.length;i++) {
            if(c[i].getName().equals("sleep")) {
                System.out.println(c[i]);
                for(int j=0;j<c[i].getParameterTypes().length;j++) {
                    System.out.println(c[i].getParameterTypes()[j]);
                }
            }
        }
        
        CtClass ccA = pool.get("gluonj.weaver.A1");
        CtMethod ccGet = ccA.getDeclaredMethod("nop");
        String body = "{try{Thread.sleep(1L);} catch(Exception e) {};setLong(3L);}";
        ccGet.setBody(body);

        Loader l = new Loader(pool);
        Class ca = l.loadClass(ccA.getName());

        Method m = ca.getMethod("nop", new Class[0]);
        Object a = ca.newInstance();
        m.invoke(a, new Object[0]);
        System.out.println(((Integer) ca.getField("x").get(a)).intValue());
    }
}