/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.pcs.PcsBehavior;
import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTMethodCall;
import gluonj.ast.pointcut.ASTNot;
import gluonj.ast.pointcut.ASTOr;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.PointcutExprUtil;
import junit.framework.TestCase;


public class TestDNFProcessor extends TestCase {

	public TestDNFProcessor() {
		super();
	}
	
	
	/*
	 * !(m1 and m2) -> !m1 or !m2
	 * !(m1 or m2) -> !m1 and !m2
	 * (m1 or m2) and (m3 or m4) -> (m1 and m3) or ((m1 and m4) or ((m2 and m3) or (m2 and m4)))
	 * (m1 and m2) and m3 -> (m1 and m2 and m3)    
	 * 
	 */
	public final void test1() {
		ASTMethodCall m1 = new ASTMethodCall(PcsBehavior.ANY);
		ASTMethodCall m2 = new ASTMethodCall(PcsBehavior.ANY);
		ASTMethodCall m3 = new ASTMethodCall(PcsBehavior.ANY);
		ASTMethodCall m4 = new ASTMethodCall(PcsBehavior.ANY);
		
		ASTPointcutExpr e1 = new ASTNot(new ASTAnd(m1, m2));
		ASTPointcutExpr e2 = new ASTNot(new ASTOr(m1, m2));
		ASTPointcutExpr e3 = new ASTAnd(new ASTOr(m1, m2), new ASTOr(m3, m4));
		ASTPointcutExpr e4 = new ASTAnd(new ASTAnd(m1, m2), m3);
		
		DisjunctiveNFProcessor processor = new DisjunctiveNFProcessor();
		
		ASTPointcutExpr new_e1 = processor.process(e1);
		ASTPointcutExpr new_e2 = processor.process(e2);
		ASTPointcutExpr new_e3 = processor.process(e3);
		ASTPointcutExpr new_e4 = processor.process(e4);
		
		ASTPointcutExpr e = new_e1;
		
		assertTrue(e instanceof ASTOr);
		assertTrue(PointcutExprUtil.getNode(e, "L") instanceof ASTNot);
		assertTrue(PointcutExprUtil.getNode(e, "R") instanceof ASTNot);
		assertTrue(PointcutExprUtil.getNode(e, "LD") == m1);
		assertTrue(PointcutExprUtil.getNode(e, "RD") == m2);
		
		e = new_e2;
		
		assertTrue(e instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "L") instanceof ASTNot);
		assertTrue(PointcutExprUtil.getNode(e, "R") instanceof ASTNot);
		assertTrue(PointcutExprUtil.getNode(e, "LD") == m1);
		assertTrue(PointcutExprUtil.getNode(e, "RD") == m2);
		
		e = new_e3;
		
		assertTrue(e instanceof ASTOr);
		assertTrue(PointcutExprUtil.getNode(e, "L") instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "LL") == m1);
		assertTrue(PointcutExprUtil.getNode(e, "LR") == m3);
		assertTrue(PointcutExprUtil.getNode(e, "R") instanceof ASTOr);
		assertTrue(PointcutExprUtil.getNode(e, "RL") instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "RLL") == m1);
		assertTrue(PointcutExprUtil.getNode(e, "RLR") == m4);
		assertTrue(PointcutExprUtil.getNode(e, "RR") instanceof ASTOr);
		assertTrue(PointcutExprUtil.getNode(e, "RRL") instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "RRR") instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "RRLL") == m2);
		assertTrue(PointcutExprUtil.getNode(e, "RRLR") == m3);
		assertTrue(PointcutExprUtil.getNode(e, "RRRL") == m2);
		assertTrue(PointcutExprUtil.getNode(e, "RRRR") == m4);
		
		e = new_e4;
		assertTrue(e instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "L") == m1);
		assertTrue(PointcutExprUtil.getNode(e, "R") instanceof ASTAnd);
		assertTrue(PointcutExprUtil.getNode(e, "RL") == m2);
		assertTrue(PointcutExprUtil.getNode(e, "RR") == m3);

	}
	

}
