/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package gluonj.weaver.gen;

import gluonj.ast.Advice;
import gluonj.ast.AdviceType;
import gluonj.ast.Aspect;
import gluonj.ast.Import;
import gluonj.ast.PointcutDecl;
import gluonj.ast.intertype.IntertypeDecl;
import gluonj.ast.java.ASTBlock;
import gluonj.ast.java.ASTFormalParameter;
import gluonj.ast.java.ASTType;
import gluonj.ast.pcs.PcsBehavior;
import gluonj.ast.pointcut.ASTAnd;
import gluonj.ast.pointcut.ASTMethodExecution;
import gluonj.ast.pointcut.ASTPCReference;
import gluonj.ast.pointcut.ASTPointcutExpr;
import gluonj.ast.pointcut.ASTThis;
import gluonj.ast.pointcut.PointcutExprUtil;
import gluonj.weaver.CannotCompileException;

import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

public class TestReferenceProcessor extends TestCase {

    public TestReferenceProcessor() {
        super();
    }

    /*
     * 
     * pointcut foo() : execution(* *.*(..));
     * 
     * before() : this(Main) && foo() { .. }
     *  
     */

    public final void test1() throws CannotCompileException {
        PointcutDecl decl = new PointcutDecl("foo", new LinkedList(),
                new ASTMethodExecution(PcsBehavior.ANY));

        ASTPointcutExpr root = new ASTAnd(new ASTThis("Main"),
                new ASTPCReference("foo", new String[0]));
        Advice adv = new Advice(new ASTFormalParameter[0], 
                AdviceType.BEFORE, root, new ASTBlock(""));

        Aspect a = new Aspect(new Import[0], new IntertypeDecl[0],
                new PointcutDecl[] { decl }, new Advice[] { adv });

        NamedPCProcessor r = new NamedPCProcessor();
        Aspect[] new_a = r.process(new Aspect[] { a });

        ASTPointcutExpr new_root = new_a[0].getAdvice()[0].getPointcutExpr();

        assertTrue(PointcutExprUtil.getNode(new_root, "L") == ((ASTAnd) root)
                .getLeft());
        assertTrue(PointcutExprUtil.getNode(new_root, "R") == decl
                .getPointcut());
    }

    /*
     * 
     * pointcut foo(String s) : execution(* *.*(..)) && this(s);
     * 
     * before(String t) : this(Main) && foo(t) { .. }
     *  
     */

    public final void test2() throws CannotCompileException {

        ASTAnd decl_pc = new ASTAnd(
                new ASTMethodExecution(PcsBehavior.ANY), new ASTThis("s",
                        "java.lang.String"));

        List param = new LinkedList();
        param.add(new ASTFormalParameter(new ASTType("java.lang.String"), "s"));
        PointcutDecl decl = new PointcutDecl("foo", param, decl_pc);

        ASTAnd root = new ASTAnd(new ASTThis("Main"), new ASTPCReference("foo",
                new String[] { "t" }));
        Advice adv = new Advice(
                new ASTFormalParameter[] { new ASTFormalParameter(new ASTType(
                        "java.lang.String"), "t") }, AdviceType.BEFORE, root, new ASTBlock(""));

        Aspect a = new Aspect(new Import[0], new IntertypeDecl[0],
                new PointcutDecl[] { decl }, new Advice[] { adv });

        NamedPCProcessor r = new NamedPCProcessor();
        Aspect[] new_a = r.process(new Aspect[] { a });

        ASTPointcutExpr new_root = new_a[0].getAdvice()[0].getPointcutExpr();

        assertTrue(PointcutExprUtil.getNode(new_root, "L") == ((ASTAnd) root)
                .getLeft());
        assertTrue(PointcutExprUtil.getNode(new_root, "RL") == decl_pc.getLeft());
        
        ASTThis t = (ASTThis) PointcutExprUtil.getNode(new_root, "RR");
        
        assertTrue(t.isBinder() && t.getVarName().equals("t"));
    }
}
