/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package test;

import gluonj.parser.ParseException;
import gluonj.ui.LoadTimeWeaver;
import gluonj.ui.StaticWeaver;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.LoaderClassPath;
import javassist.NotFoundException;
import junit.framework.TestCase;
import junit.framework.TestResult;

public class GluonJTestCase extends TestCase {

    
    public static final String TESTDIR = new File(".").getAbsolutePath()
            + File.separator + "test" + File.separator;

    protected String RUNNER = "gluonj.test.Runner";

    private ClassPool pool;

    public void run(TestResult result) {
        pool = new ClassPool();
        pool.appendClassPath(new LoaderClassPath(ClassLoader
                .getSystemClassLoader()));

        super.run(result);
    }

    protected ClassPool getPool() {
        if (pool == null) {
            throw new RuntimeException(
                    "GluonJTestCase.getPool() must be called only in test methods.");
        } else {
            return pool;
        }
    }

    protected String getTestingMethod() {
        try {
            Class cTestCase = Class.forName("junit.framework.TestCase");
            Field[] fs = cTestCase.getDeclaredFields();
            Field ffName = cTestCase.getDeclaredField("fName");
            ffName.setAccessible(true);
            return (String) ffName.get(this);
        } catch (SecurityException e) {
            throw new RuntimeException(e);
        } catch (NoSuchFieldException e) {
            throw new RuntimeException(e);
        } catch (IllegalArgumentException e) {
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    protected void dump(String[] targetClassName,
            String[] aspects, final String dist) throws ParseException,
            gluonj.weaver.CannotCompileException {

        String packageName = this.getClass().getPackage().getName();
        
        String aspectDir = TESTDIR
                + packageName.replace('.', File.separatorChar) + File.separator;

        for (int i = 0; i < aspects.length; i++) {
            aspects[i] = aspectDir + aspects[i];
        }

        for (int i = 0; i < targetClassName.length; i++) {
            targetClassName[i] = packageName + "." + targetClassName[i];
        }

        StaticWeaver.ClassWritingHandler handler = new StaticWeaver.ClassWritingHandler() {
            public void onWrite(CtClass cc) throws NotFoundException,
                    IOException, CannotCompileException {
                cc.writeFile(dist);
            }

            public void finish() throws IOException {
            }
        };

        StaticWeaver.weave(aspects, targetClassName, getPool(), handler);
    }

    protected void runOnGluonJ(String className, String methodName,
            String[] aspects) throws Throwable {
        
        String packageName = this.getClass().getPackage().getName();
        
        String aspectDir = TESTDIR
                + packageName.replace('.', File.separatorChar) + File.separator;

        for (int i = 0; i < aspects.length; i++) {
            aspects[i] = aspectDir + aspects[i];
        }

        try {
            LoadTimeWeaver.run(aspects, RUNNER, new String[] {
                    packageName + "." + className, methodName }, getPool());
        } catch (Throwable t) {
            System.err.println(t);
            throw t;
        }
    }
}