/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package test.injection;

public class Main {

    public void testPerobjects() {

        Digit b1 = new Digit();
        Digit b2 = new Digit();
        Digit b3 = new Digit();

        Object o12 = getAssociatedWith(b1, b2);
        Object o13 = getAssociatedWith(b1, b3);
        Object o23 = getAssociatedWith(b2, b3);

        if (o12 == null || o13 == null || o23 == null) {
            throw new RuntimeException(
                    "an aspect instance is not associated with pair of objects.");
        }

        if (o12.hashCode() == o23.hashCode()) {
            throw new RuntimeException(
                    "same aspect instance is associated with different pair of objects.");
        }
        if (o12.hashCode() == o13.hashCode()) {
            throw new RuntimeException(
                    "same aspect instance is associated with different pair of objects.");
        }
    }

    private Adder getAssociatedWith(Digit b1, Digit b2) {
        return null;
    }

    public void testPerobjectsInit() {

        Digit b1 = new Digit(10);
        Digit b2 = new Digit(20);
        Digit b3 = new Digit(30);

        Adder o12 = getAssociatedWith(b1, b2);
        Adder o13 = getAssociatedWith(b1, b3);

        /*
         * b1 = 10 + 10 * 20 + 10 * 30, b2 = 20 + 10 * 20, b3 = 3 + 10 * 30;
         */
        doAdd(b1, b2);
        doAdd(b1, b3);

        if (b1.getValue() != 510) {
            throw new RuntimeException(
                    "an aspect instance is not correctly associated with objects.");
        }
        if (b2.getValue() != 220) {
            throw new RuntimeException(
                    "an aspect instance is not correctly associated with objects.");
        }
        if (b3.getValue() != 330) {
            throw new RuntimeException(
                    "an aspect instance is not correctly associated with objects.");
        }
    }

    private void doAdd(Digit b1, Digit b3) {
    }

    public void testPercflow() {
        if (getDigit() != null) {
            throw new RuntimeException(
                    "an aspect instance is associated with object even out of specifiled control flow.");
        }
        Digit result = fact(3);
        if (result.getValue() != 6) {
            throw new RuntimeException(
                    "an aspect instance is not associated with each control flow.");
        }
    }

    Digit fact(int i) {
        if(i < 1) {
            return new Digit(1);
        }
        Digit d = getDigit();
        d.setValue(i);
        Digit d2 = fact(i - 1);
        return new Digit(d.getValue() * d2.getValue());
    }

    Digit getDigit() {
        return null;
    }

    public void testPerobject() {

        Digit d1 = new Digit(1);
        Digit d2 = new Digit(2);
        Adder a1 = getAspectOf(d1);
        Adder a2 = getAspectOf(d2);

        if (a1 == null || a2 == null) {
            throw new RuntimeException(
                    "an aspect instance is not associated with object.");
        }
        if (a1.hashCode() == a2.hashCode()) {
            throw new RuntimeException(
                    "same aspect instance is associated with different objects.");
        }

    }

    public Adder getAspectOf(Digit d) {
        return null;
    }
    
    public void testStaticField() {

    }
}