/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package test.pc.shadow;

import gluonj.runtime.ClosureStack;

public class Main {

    private int count;

    private int weaved_before;

    private int weaved_after;

    public void testBeforeExecution() {
        count();
        if (weaved_before != 1) {
            throw new RuntimeException(
                    "before execution advice is not weaved. weaved = "
                            + weaved_before);
        }
        if (weaved_after != 1) {
            throw new RuntimeException(
                    "after execution advice is not weaved. weaved = "
                            + weaved_after);
        }
    }

    public void testBeforeCall() {
        count();
        if (weaved_before != 1) {
            throw new RuntimeException(
                    "before call advice is not weaved. weaved = "
                            + weaved_before);
        }
        if (weaved_after != 1) {
            throw new RuntimeException(
                    "after call advice is not weaved. weaved = " + weaved_after);
        }
    }

    public void testBeforeSet() {
        count();
        if (weaved_before != 1) {
            throw new RuntimeException(
                    "before set advice is not weaved. weaved = "
                            + weaved_before);
        }
        if (weaved_after != 1) {
            throw new RuntimeException(
                    "after set advice is not weaved. weaved = " + weaved_after);
        }
    }

    public void testBeforeGet() {
        count();
        if (weaved_before != 1) {
            throw new RuntimeException(
                    "before get advice is not weaved. weaved = "
                            + weaved_before);
        }
        if (weaved_after != 1) {
            throw new RuntimeException(
                    "after get advice is not weaved. weaved = " + weaved_after);
        }
    }

    public void testAroundExecution() {
        count();
        if (weaved_before != 1 || weaved_after != 1) {
            throw new RuntimeException("around execution advice is not weaved.");
        }
        if (count != 1) {
            throw new RuntimeException(
                    "execution joinpoint is not overwritten by around advice.");
        }
    }

    public void testAroundCall() {
        count();
        if (weaved_before != 1 || weaved_after != 1) {
            throw new RuntimeException("around call advice is not weaved.");
        }
        if (count != 1) {
            throw new RuntimeException(
                    "call joinpoint is not overwritten by around advice.");
        }
    }

    public void testAroundSet() {
        count();
        if (weaved_before != 1 || weaved_after != 1) {
            throw new RuntimeException("around set advice is not weaved.");
        }
        if (count != 1) {
            throw new RuntimeException(
                    "set joinpoint is not overwritten by around advice.");
        }
    }

    public void testAroundGet() {
        count();
        if (weaved_before != 1 || weaved_after != 1) {
            throw new RuntimeException("around get advice is not weaved.");
        }
        if (count != 1) {
            throw new RuntimeException(
                    "get joinpoint is not overwritten by around advice.");
        }
    }

    public void count() {
        // <- before execution(count()) advice
        Number n = new Number();
        // <- before call(count()) advice
        int i = n.one(1);
        // <- after call(count()) advice
        // <- before get(count) advice
        int j = i + count;
        // <- after get(count) advice
        // <- before set(count) advice
        count = j;
        // <- after set(count) advice
        // <- after execution(count()) advice
    }

    public static Object proceed() {
        try {
            return ClosureStack.peek().proceed();
        } catch (Throwable t) {
            return null;
        }
    }
}