/*
 * GluonJ 2005, an aspect-oriented system for Java.
 * Copyright (C) 2006 Rei Ishikawa.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 */
package test.pc.shadow.execution;

import test.NotMatchedException;
import test.pc.shadow.sample.A;
import test.pc.shadow.sample.AB;
import test.pc.shadow.sample.B;

public class Main {

    /*
     * using wildcards "*" in the name of methods.
     */
    public void testExecutionWildcard1() {

        int flagnum = 3;
        boolean[][] flagSet = makeFlagSet(flagnum);
        for (int i = 0; i < Math.pow(2, flagnum) - 1; i++) {
            boolean thrown = false;
            try {
                testExecutionWildcard1(flagSet[i]);
            } catch (NotMatchedException e) {
                thrown = true;
            }
            if (!thrown) {
                throw new RuntimeException();
            }
        }
        testExecutionWildcard1(flagSet[(int) Math.pow(2, flagnum) - 1]);

    }

    private boolean[][] makeFlagSet(int size) {

        boolean[][] flags = new boolean[(int) Math.pow(2, size)][size];
        for (int i = 0; i < Math.pow(2, size); i++) {
            for (int j = 0; j < size; j++) {
                flags[i][j] = ((i >> j) % 2 != 0);
            }
        }
        return flags;
    }

    public int testExecutionWildcard1_flagnum = 3;

    public void testExecutionWildcard1(boolean[] flags) {

        A a = new A();
        if (flags[0])
            a.one();
        if (flags[1])
            a.two();
        if (flags[2])
            a.oneplustwo();

        if (a.before_one != 2 || a.after_one != 2) {
            throw new NotMatchedException(
                    "execution(void A.one*()) does not specify appropriate joinpoints. A=" + a);
        }
        if (a.before_two != 2 || a.after_two != 2) {
            throw new NotMatchedException(
                    "execution(void A.*two()) does not specify appropriate joinpoints. A=" + a);
        }
        if (a.before_all != 3 || a.after_all != 3) {
            throw new NotMatchedException(
                    "execution(void A.*o*()) does not specify appropriate joinpoints. A=" + a);
        }
    }

    public int testExecutionWildcard2_flagnum = 4;

    /*
     * using wildcards "*" ".." in the name of method types.
     */
    public void testExecutionWildcard2(boolean[] flags) {
        A a = new A();
        B b = new B();
        AB ab = new AB();
        test.pc.shadow.sample.sub.A suba = new test.pc.shadow.sample.sub.A();

        if (flags[0])
            a.one();
        if (flags[1])
            b.one();
        if (flags[2])
            ab.one();
        if (flags[3])
            suba.one();

        if (a.before_one != 2 || a.after_one != 2 || ab.before_one != 2
                || ab.after_one != 2) {
            throw new NotMatchedException(
                    "execution(void A*.one()) does not specify appropriate joinpoints.");
        }
        if (b.before_one != 1 || b.after_one != 1 || ab.before_one != 2
                || ab.after_one != 2) {
            throw new NotMatchedException(
                    "execution(void *B.one()) does not specify appropriate joinpoints.");
        }
        if (a.before_one != 2 || a.after_one != 2 || suba.before_one != 1
                || suba.after_one != 1) {
            throw new NotMatchedException(
                    "execution(void *..A.one()) does not specify appropriate joinpoints.");
        }
    }

    public int testExecutionWildcard3_flagnum = 4;

    /*
     * using wildcards "*" ".." in the name of return types of methods.
     */
    public void testExecutionWildcard3(boolean[] flags) {
        A a = new A();
        if (flags[0])
            a.getA();
        if (flags[1])
            a.getB();
        if (flags[2])
            a.getAB();
        if (flags[3])
            a.getSubA();

        if (a.before_return_A != 2 || a.after_return_A != 2) {
            throw new NotMatchedException(
                    "execution(A* A.*()) does not specify appropriate joinpoints.");
        }
        if (a.before_return_B != 2 || a.after_return_B != 2) {
            throw new NotMatchedException(
                    "execution(*B A.*()) does not specify appropriate joinpoints.");
        }
        if (a.before_return_subA != 2 || a.after_return_subA != 2) {
            throw new NotMatchedException(
                    "execution(*..A A.*()) does not specify appropriate joinpoints.");
        }

    }
}